@@ -1,5 +1,4 @@
 # acl-lib.pl
-	
 # Library for editing webmin users, passwords and access rights
 
 do '../web-lib.pl';
@@ -10,9 +9,8 @@ do '../web-lib.pl';
 # Returns a list of hashes containing user details
 sub list_users
 {
-local(@mods, %miniserv, $_, @rv, %acl);
+local(%miniserv, $_, @rv, %acl);
 &read_acl(undef, \%acl);
-@mods = &list_modules();
 &get_miniserv_config(\%miniserv);
 open(PWFILE, $miniserv{'userfile'});
 while(<PWFILE>) {
@@ -65,28 +63,18 @@ return @rv;
 }
 
 # list_modules()
-# Returns a list of all modules available on this system
+# Returns a list of the dirs of all modules available on this system
 sub list_modules
 {
-local(@rv, $d);
-opendir(DIR, "..");
-foreach $d (readdir(DIR)) {
-	local %minfo;
-	if ($d !~ /^\./ && (%minfo = &get_module_info($d)) &&
-	    &check_os_support(\%minfo)) {
-		push(@rv, \%minfo);
-		}
-	}
-closedir(DIR);
-return map { $_->{'dir'} } sort { $a->{'desc'} cmp $b->{'desc'} } @rv;
+return map { $_->{'dir'} } &list_module_infos();
 }
 
-# module_info(module)
-# Returns an array of  module, name, desc, [os_support]
-sub module_info
+# list_module_infos()
+# Returns a list of the details of all  modules available on this system
+sub list_module_infos
 {
-local %module = &get_module_info($_[0]);
-return ($_[0], $module{'name'}, $module{"desc"}, $module{'os_support'});
+local @mods = grep { &check_os_support($_) } &get_all_module_infos();
+return sort { $a->{'desc'} cmp $b->{'desc'} } @mods;
 }
 
 # create_user(&details, clone)
@@ -101,7 +89,7 @@ if ($user{'theme'}) {
 	$miniserv{"preroot_".$user{'name'}} = $user{'theme'};
 	}
 elsif (defined($user{'theme'})) {
-	$miniserv{"preroot_".$user{'name'}} = $miniserv{'root'};
+	$miniserv{"preroot_".$user{'name'}} = "";
 	}
 &put_miniserv_config(\%miniserv);
 &unlock_file($ENV{'MINISERV_CONFIG'});
@@ -137,7 +125,7 @@ $gconfig{"theme_".$user{'name'}} = $user{'theme'} if (defined($user{'theme'}));
 &write_file("$config_directory/config", \%gconfig);
 
 if ($_[1]) {
-	foreach $m (@mods) {
+	foreach $m ("", @mods) {
 		local $file = "$config_directory/$m/$_[1].acl";
 		local $dest = "$config_directory/$m/$user{'name'}.acl";
 		if (-r $file) {
@@ -157,12 +145,14 @@ local(%user, %miniserv, @pwfile, @acl, @mods, $_, $m);
 
 &lock_file($ENV{'MINISERV_CONFIG'});
 &get_miniserv_config(\%miniserv);
-delete($miniserv{"preroot_".$_[0]});
 if ($user{'theme'}) {
 	$miniserv{"preroot_".$user{'name'}} = $user{'theme'};
 	}
 elsif (defined($user{'theme'})) {
-	$miniserv{"preroot_".$user{'name'}} = $miniserv{'root'};
+	$miniserv{"preroot_".$user{'name'}} = "";
+	}
+else {
+	delete($miniserv{"preroot_".$_[0]});
 	}
 &put_miniserv_config(\%miniserv);
 &unlock_file($ENV{'MINISERV_CONFIG'});
@@ -173,7 +163,7 @@ open(PWFILE, $miniserv{'userfile'});
 close(PWFILE);
 open(PWFILE, "> $miniserv{'userfile'}");
 foreach (@pwfile) {
-	if (/^([^:\s]+):([^:\s]+)/ && $1 eq $_[0]) {
+	if (/^([^:]+):/ && $1 eq $_[0]) {
 		print PWFILE "$user{'name'}:$user{'pass'}:",
 			     "$user{'sync'}:$user{'cert'}:",
 			$user{'allow'} ? "allow $user{'allow'}" :
@@ -252,7 +242,7 @@ open(PWFILE, $miniserv{'userfile'});
 close(PWFILE);
 open(PWFILE, "> $miniserv{'userfile'}");
 foreach (@pwfile) {
-	if (!/^([^:\s]+):([^:\s]+)/ || $1 ne $_[0]) { print PWFILE $_; }
+	if (!/^([^:]+):/ || $1 ne $_[0]) { print PWFILE $_; }
 	}
 close(PWFILE);
 &unlock_file($miniserv{'userfile'});
@@ -263,7 +253,7 @@ open(ACL, &acl_filename());
 close(ACL);
 open(ACL, "> ".&acl_filename());
 foreach (@acl) {
-	if (!/^(\S+):/ || $1 ne $_[0]) { print ACL $_; }
+	if (!/^([^:]+):/ || $1 ne $_[0]) { print ACL $_; }
 	}
 close(ACL);
 &unlock_file(&acl_filename());
@@ -291,7 +281,7 @@ if ($miniserv{'session'}) {
 	}
 }
 
-# create_group(&group)
+# create_group(&group, [clone])
 # Add a new webmin group
 sub create_group
 {
@@ -300,6 +290,18 @@ open(GROUP, ">>$config_directory/webmin.groups");
 print GROUP &group_line($_[0]),"\n";
 close(GROUP);
 &unlock_file("$config_directory/webmin.groups");
+
+if ($_[1]) {
+	foreach $m ("", &list_modules()) {
+		local $file = "$config_directory/$m/$_[1].acl";
+		local $dest = "$config_directory/$m/$_[0]->{'name'}.acl";
+		if (-r $file) {
+			local %macl;
+			&read_file($file, \%macl);
+			&write_file($dest, \%macl);
+			}
+		}
+	}
 }
 
 # modify_group(name, &group)
@@ -359,6 +361,9 @@ return "$user{'name'}: ".join(' ', @{$user{'modules'}})."\n";
 sub can_edit_user
 {
 return 1 if ($access{'users'} eq '*');
+if ($access{'users'} eq '~') {
+	return $base_remote_user eq $_[0];
+	}
 local $u;
 local $glist = $_[1] ? $_[1] : [ &list_groups() ];
 foreach $u (split(/\s+/, $access{'users'})) {
@@ -391,6 +396,75 @@ if ($@) {
 	}
 }
 
+# update_members(&allusers, &allgroups, &modules, &members)
+# Update the members users and groups of some group
+sub update_members
+{
+foreach $m (@{$_[3]}) {
+	local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
+	if ($u) {
+		$u->{'modules'} = [ @{$_[2]}, @{$u->{'ownmods'}} ];
+		&modify_user($u->{'name'}, $u);
+		}
+	local ($g) = grep { $_->{'name'} eq $m } @{$_[1]};
+	if ($g) {
+		$g->{'modules'} = [ @{$_[2]}, @{$g->{'ownmods'}} ];
+		&modify_group($g->{'name'}, $g);
+		&update_members($_[0], $_[1], $_[2], $g->{'members'});
+		}
+	}
+}
+
+# copy_acl_files(from, to, &modules)
+# Copy all .acl files from some user or group to another in a list of modules
+sub copy_acl_files
+{
+local $m;
+foreach $m (@{$_[2]}) {
+	unlink("$config_directory/$m/$_[1].acl");
+	local %acl;
+	if (&read_file("$config_directory/$m/$_[0].acl", \%acl)) {
+		&write_file("$config_directory/$m/$_[1].acl", \%acl);
+		}
+	}
+}
+
+# set_acl_files(&allusers, &allgroups, module, &members, &access)
+sub set_acl_files
+{
+foreach $m (@{$_[3]}) {
+	local ($u) = grep { $_->{'name'} eq $m } @{$_[0]};
+	local ($g) = grep { $_->{'name'} eq $m } @{$_[1]};
+	if ($u || $g) {
+		local $n = $u ? $u->{'name'} : $g->{'name'};
+		&lock_file("$config_directory/$_[2]/$n.acl");
+		&write_file("$config_directory/$_[2]/$n.acl", $_[4]);
+		chmod(0640, "$config_directory/$_[2]/$n.acl");
+		&unlock_file("$config_directory/$_[2]/$n.acl");
+		}
+	if ($g) {
+		&set_acl_files($_[0], $_[1], $_[2], $g->{'members'}, $_[4]);
+		}
+	}
+}
+
+# get_ssleay()
+# Returns the path to OpenSSL or equivalent
+sub get_ssleay
+{
+if (&has_command($config{'ssleay'})) {
+	return &has_command($config{'ssleay'});
+	}
+elsif (&has_command("openssl")) {
+	return &has_command("openssl");
+	}
+elsif (&has_command("ssleay")) {
+	return &has_command("ssleay");
+	}
+else {
+	return undef;
+	}
+}
 
 1;
 
