@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './acl-lib.pl';
+do 'acl-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -36,6 +36,10 @@ elsif ($action eq 'acl') {
 	return &text('log_acl', "<tt>$object</tt>",
 		     "<i>".&html_escape($p->{'moddesc'})."</i>");
 	}
+elsif ($action eq 'reset') {
+	return &text('log_reset', "<tt>$object</tt>",
+		     "<i>".&html_escape($p->{'moddesc'})."</i>");
+	}
 elsif ($action eq 'cert') {
 	return &text('log_cert', "<tt>".&html_escape($object)."</tt>");
 	}
