@@ -1,5 +1,5 @@
 
-require './at-lib.pl';
+require 'at-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the at module
