@@ -1,3 +1,4 @@
+#!/usr/local/bin/perl
 # copyconfig.pl
 # Copy the appropriate config file for each module into the webmin config
 # directory. If it is already there, merge in new directives. Called with
@@ -28,6 +29,9 @@ foreach $m (@mods) {
 	elsif (-r "$srcdir/config-$os") {
 		$conf = "$srcdir/config-$os";
 		}
+	elsif ($os =~ /^(\S+)-(\S+)$/ && -r "$srcdir/config-*-$2") {
+		$conf = "$srcdir/config-*-$2";
+		}
 	elsif (-r "$srcdir/config") {
 		$conf = "$srcdir/config";
 		}
@@ -76,7 +80,7 @@ local($arr);
 $arr = $_[1];
 open(ARFILE, $_[0]) || return 0;
 while(<ARFILE>) {
-        chop;
+	s/\r|\n//g;
         if (!/^#/ && /^([^=]+)=(.*)$/) { $$arr{$1} = $2; }
         }
 close(ARFILE);
