@@ -1,14 +1,20 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './cron-lib.pl';
+do 'cron-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params, [long])
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
 local ($user, $script, $action, $type, $object, $p, $long) = @_;
-if ($action eq 'modify') {
+if ($type eq "env") {
+	return &text('log_env_'.$action, "<tt>$object</tt>");
+	}
+elsif ($action eq "move") {
+	return &text('log_move', "<tt>$object</tt>");
+	}
+elsif ($action eq 'modify') {
 	return &text($long ? 'log_modify_l' : 'log_modify',
 		     "<tt>$object</tt>",
 		     "<tt>".&html_escape($p->{'cmd'})."</tt>");
