@@ -9,13 +9,11 @@ do '../web-lib.pl';
 sub get_parent_config
 { 
 return $get_parent_config_cache if ($get_parent_config_cache);
-return ($get_parent_config_cache =
-		{
-		 'file' => $config{'dhcpd_conf'},
-	     'members' => &get_config(),
-	     'line' => -1,
-	     'eline' => $get_config_lines 
-		  } );
+return $get_parent_config_cache = {
+	 'file' => $config{'dhcpd_conf'},
+	 'members' => &get_config(),
+	 'line' => -1,
+	 'eline' => $get_config_lines };
 }
 
 # get_config()
@@ -23,18 +21,44 @@ return ($get_parent_config_cache =
 sub get_config
 {
 return \@get_config_cache if (@get_config_cache);
-local(@tok, @rv, $i, $t, $j, $ifile, @inc, $str);
-
-$get_config_lines = &tokenize_file($config{'dhcpd_conf'}, \@tok);
-$i = 0; $j = 0;
-while($i < @tok) {
-	$str = &parse_struct(\@tok, \$i, $j++, $config{'dhcpd_conf'});
-	if ($str) { push(@rv, $str); }
-	}
+local @rv = &get_config_file($config{'dhcpd_conf'}, \$get_config_lines);
 @get_config_cache = @rv;
 return \@get_config_cache;
 }
 
+# get_config_file(file, [&lines])
+sub get_config_file
+{
+local (@tok, $i, $j, @rv, $c);
+$i = 0; $j = 0;
+local $lines = &tokenize_file($_[0], \@tok);
+${$_[1]} = $lines if ($_[1]);
+while($i < @tok) {
+	local $str = &parse_struct(\@tok, \$i, $j++, $_[0]);
+	if ($str) {
+		if ($str->{'name'} eq 'include') {
+			# Expand the include directive
+			local $p = $str->{'values'}->[0];
+			if ($p !~ /^\//) {
+				$config{'dhcpd_conf'} =~ /^(\S+)\//;
+				$p = "$1/$p";
+				}
+			local @inc = &get_config_file($p);
+			$j--;
+			foreach $c (@inc) {
+				$c->{'index'} += $j;
+				}
+			push(@rv, @inc);
+			$j += scalar(@inc);
+			}
+		else {
+			push(@rv, $str);
+			}
+		}
+	}
+return @rv;
+}
+
 # tokenize_file(file, &tokens)
 sub tokenize_file
 {
@@ -46,8 +70,10 @@ while($line = <FILE>) {
 	$line =~ s/\r|\n//g;
 	$line =~ s/^([^"#]*)#.*$/$1/g;
 	$line =~ s/^([^"]*)\/\/.*$/$1/g;
-	$line =~ s/\s+#.*$//g;
-	$line =~ s/\\"/\\176/g;
+	$line =~ s/^([^"]*)\s+#.*$/$1/g;	# remove stuff after #, unless
+	$line =~ s/^(.*".*".*)\s+#.*$/$1/g;	#  it is inside quotes
+	$line =~ s/\\\\/\\134/g;		# convert \\ into \134
+	$line =~ s/([^\\])\\"/$1\\042/g;	# convert escaped quotes to \042
 	while(1) {
 		if (!$cmode && $line =~ /\/\*/ && $line !~ /\".*\/\*.*\"/) {
 			# start of a C-style comment
@@ -81,6 +107,7 @@ while($line = <FILE>) {
 	$lines++;
 	}
 close(FILE);
+#print STDERR "tokenized $_[0] into $lines\n";
 return $lines;
 }
 
@@ -90,10 +117,18 @@ return $lines;
 sub parse_struct
 {
 local(%str, $i, $t, @vals, @quotes, $str, @text);
+local $lref = &read_file_lines($_[3]);
 $i = ${$_[1]};
 $str{'name'} = $_[0]->[$i]->[0];
 $str{'line'} = $_[0]->[$i]->[2];
+if ($str{'line'} && $lref->[$str{'line'}-1] =~ /^\s*(#|\/\/)\s*(.*)/) {
+	# Previous line is a comment, so include it in the directive
+	$str{'line'}--;
+	$str{'comment'} = $2;
+	}
+#print STDERR "parsing at line $str{'line'} = $str{'name'}\n";
 while(1) {
+	# Add values between directive name and { or ;
 	$t = $_[0]->[++$i];
 	if ($t->[0] eq "{" || $t->[0] eq ";") { last; }
 	elsif (!defined($t->[0])) { ${$_[1]} = $i; return undef; }
@@ -106,6 +141,7 @@ $str{'value'} = $vals[0];
 $str{'text'} = join(" ", @text);
 $str{'index'} = $_[2];
 $str{'file'} = $_[3];
+$str{'fline'} = $_[0]->[$i]->[2];
 if ($t->[0] eq "{") {
 	# contains sub-structures.. parse them
 	local(@mems, $j);
@@ -116,7 +152,10 @@ if ($t->[0] eq "{") {
 		if (!defined($_[0]->[$i]->[0]))
 			{ ${$_[1]} = $i; return undef; }
 		$str = &parse_struct($_[0], \$i, $j++, $_[3]);
-		if ($str) { push(@mems, $str); }
+		if ($str) {
+			$str->{'parent'} = \%str;
+			push(@mems, $str);
+			}
 		}
 	$str{'members'} = \@mems;
 	$i++;		# skip trailing }
@@ -169,6 +208,20 @@ for($i=3; $i<@_; $i+=2) {
 return $rv."</td>\n";
 }
 
+# wide_choice_input(text, name, &config, [display, option]+)
+sub wide_choice_input
+{
+local($rv, $v, $i, @ops);
+$rv = "<td><b>$_[0]</b></td> <td colspan=3>";
+$v = &find_value($_[1], $_[2]);
+for($i=3; $i<@_; $i+=2) {
+	@ops = split(/,/, $_[$i+1]);
+	$rv .= "<input type=radio name=$_[1] value=\"$ops[0]\" ".
+		($v eq $ops[0] ? "checked" : "").">$_[$i]\n";
+	}
+return $rv."</td>\n";
+}
+
 # save_choice(name, &parent, indent)
 sub save_choice
 {
@@ -285,13 +338,14 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		# the parent
 		$lref = &read_file_lines($_[0]->{'file'});
 		@nl = &directive_lines($newv[$i], $_[3]);
-		splice(@$lref, $_[0]->{'line'}+1, 0, @nl);
-		&renumber(&get_config(), $_[0]->{'line'}+1,
+		$nline = $_[0]->{'fline'}+1;
+		splice(@$lref, $nline, 0, @nl);
+		&renumber(&get_config(), $nline,
 			  $_[0]->{'file'}, scalar(@nl));
 		&renumber_index($_[0]->{'members'}, 0, 1);
 		$newv[$i]->{'file'} = $_[0]->{'file'};
-		$newv[$i]->{'line'} = $_[0]->{'line'}+1;
-		$newv[$i]->{'eline'} = $_[0]->{'line'} + scalar(@nl);
+		$newv[$i]->{'line'} = $nline;
+		$newv[$i]->{'eline'} = $nline + scalar(@nl);
 		unshift(@$pm, $newv[$i]);
 		}
 	elsif ($i >= @oldv) {
@@ -322,9 +376,9 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		$lref = &read_file_lines($oldv[$i]->{'file'});
 		@nl = &directive_lines($newv[$i], $_[3]);
 		$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
+		splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
 		&renumber(&get_config(), $oldv[$i]->{'eline'},
 			  $oldv[$i]->{'file'}, scalar(@nl) - $ol);
-		splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
 		$newv[$i]->{'file'} = $_[0]->{'file'};
 		$newv[$i]->{'line'} = $oldv[$i]->{'line'};
 		$newv[$i]->{'eline'} = $oldv[$i]->{'line'} + scalar(@nl) - 1;
@@ -338,16 +392,20 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 sub directive_lines
 {
 local(@rv, $v, $m, $i);
-$rv[0] = "\t" x $_[1];
-$rv[0] .= "$_[0]->{'name'}";
+if ($_[0]->{'comment'}) {
+	push(@rv, ("\t" x $_[1])."# ".$_[0]->{'comment'});
+	}
+local $first = "\t" x $_[1];
+$first .= "$_[0]->{'name'}";
 for($i=0; $i<@{$_[0]->{'values'}}; $i++) {
 	$v = $_[0]->{'values'}->[$i];
-	if ($_[0]->{'quotes'}->[$i]) { $rv[0] .= " \"$v\""; }
-	else { $rv[0] .= " $v"; }
+	if ($_[0]->{'quotes'}->[$i]) { $first .= " \"$v\""; }
+	else { $first .= " $v"; }
 	}
+push(@rv, $first);
 if ($_[0]->{'type'}) {
 	# multiple values.. include them as well
-	$rv[0] .= " {";
+	$rv[$#rv] .= " {";
 	foreach $m (@{$_[0]->{'members'}}) {
 		push(@rv, &directive_lines($m, $_[1]+1));
 		}
@@ -413,12 +471,16 @@ if ($d1->{'type'}) {
 return 0;
 }
 
-# group_name($members)
+# group_name(&members, &group)
 sub group_name
 {
-return ($_[0] == 0) ? $text{'index_nomemb'} :
-	($_[0] == 1) ? $text{'index_1memb'} :
-	($_[0] >= 2 && $_[0] <= 4) ? &text('index_234memb', $_[0]) :
+local @opts = &find("option", $_[1]->{'members'});
+local ($dn) = grep { $_->{'values'}->[0] eq 'domain-name' } @opts;
+return  $config{'group_name'} && $dn ? &text('index_gdom',$dn->{'values'}->[1]):
+	$_[1]->{'values'}->[0] ? $_[1]->{'values'}->[0] :
+        $_[0] == 0 ? $text{'index_nomemb'} :
+	$_[0] == 1 ? $text{'index_1memb'} :
+	$_[0] >= 2 && $_[0] <= 4 ? &text('index_234memb', $_[0]) :
 	&text('index_memb', $_[0]);
 
 }
@@ -593,12 +655,17 @@ if ($node ne get_parent_config()) {
 	if (($acc->{'smode'} == 2) || ($smode == 2) ||
 	    ($acc->{'smode'} == 3) || ($smode == 3)) {
 		# check parents
-		$parnode=&get_parent_config();								
-		foreach ($in{'sidx'}, $in{'uidx'}, $in{'gidx'}) {
-			if ($_ ne '') {
-				$parnode = $parnode->{'members'}->[$_];
-				return 0 unless &can_node(\@perm, $acc, $parnode);
-				}
+		#$parnode=&get_parent_config();
+		#foreach ($in{'sidx'}, $in{'uidx'}, $in{'gidx'}) {
+		#	if ($_ ne '') {
+		#		$parnode = $parnode->{'members'}->[$_];
+		#		return 0 unless &can_node(\@perm, $acc, $parnode);
+		#		}
+		#	}
+		$parnode = $node->{'parent'};
+		while($parnode) {
+			return 0 unless &can_node(\@perm, $acc, $parnode);
+			$parnode = $parnode->{'parent'};
 			}
 		}
 		
@@ -619,7 +686,14 @@ local ($rperm, $acc, $node)=@_;
 # per-object permissions
 local $otype=$obj_names2types{$node->{'name'}};
 if ($acc->{'per_' . $otype . '_acls'}) {  
-	local $acl = $acc->{'ACL' . $otype . '_' . $node->{'values'}->[0]};
+	local $name = $node->{'values'}->[0];
+	if (!$name && $node->{'name'} eq 'group') {
+		local @opts = &find("option", $node->{'members'});
+		local ($dn) = grep { $_->{'values'}->[0] eq 'domain-name' }
+				   @opts;
+		$name = $dn->{'values'}->[1] if ($dn);
+		}
+	local $acl = $acc->{'ACL' . $otype . '_' . $name};
 	foreach (@{$rperm}) {
 		next if $_ eq 'c'; # skip creation perms for per-obj acls
 		return 0 if index($acl, $_) == -1;
@@ -659,4 +733,89 @@ undef($can_cache);
 return &save_module_acl($_[1]);
 }
 
+# find_recursive(name, &config, [parent])
+# Returns a list of all config entries with some name, no matter where they
+# are in the heirarchy
+sub find_recursive
+{
+local ($c, @rv);
+foreach $c (@{$_[1]}) {
+	if ($c->{'name'} eq $_[0]) {
+		push(@rv, $c);
+		}
+	if ($c->{'type'}) {
+		push(@rv, &find_recursive($_[0], $c->{'members'}, $c));
+		}
+	}
+return @rv;
+}
+
+# find_parents(&object)
+sub find_parents
+{
+local ($gidx, $uidx, $sidx);
+local $p = $_[0]->{'parent'};
+while($p) {
+	$gidx = $p->{'index'} if ($p->{'name'} eq 'group');
+	$uidx = $p->{'index'} if ($p->{'name'} eq 'subnet');
+	$sidx = $p->{'index'} if ($p->{'name'} eq 'shared-network');
+	$p = $p->{'parent'};
+	}
+return ($gidx, $uidx, $sidx);
+}
+
+# get_dhcpd_version(&out)
+sub get_dhcpd_version
+{
+local $out = `$config{'dhcpd_path'} -v 2>&1`;
+${$_[0]} = $out;
+return undef if ($out !~ /DHCP/ || $out =~ /V1/);
+return $out =~ /\sV([0-9\.]+)/ ? $1 :
+       $out =~ /-T/ ? 3 : 2;
+}
+
+# restart_dhcpd()
+# Re-starts the DHCP server, and returns an error message if something fails
+sub restart_dhcpd
+{
+local $out;
+if ($config{'restart_cmd'}) {
+	# Run the restart script
+	$out = &backquote_logged("$config{'restart_cmd'} 2>&1");
+	return "<pre>$out</pre>" if ($?);
+	}
+else {
+	# Kill and re-run the server
+	local $pid;
+	if (open(PID, $config{'pid_file'})) {
+		chop($pid = <PID>);
+		close(PID);
+		}
+	$pid && &kill_logged('TERM', $pid) ||
+		return "$text{'restart_errmsg2'} $pid : $!";
+	if ($config{'start_cmd'}) {
+		$out = &backquote_logged("$config{'start_cmd'} 2>&1");
+		}
+	else {
+		$out = &backquote_logged("$config{'dhcpd_path'} -cf $config{'dhcpd_conf'} -lf $config{'lease_file'} $config{'interfaces'} 2>&1");
+		}
+	return "<pre>$out</pre>" if ($?);
+	}
+return undef;
+}
+
+# search_re(value, match)
+sub search_re
+{
+if ($in{'match'} == 0) {
+	return lc($_[0]) eq lc($_[1]);
+	}
+elsif ($in{'match'} == 1) {
+	return $_[0] =~ /\Q$_[1]\E/i;
+	}
+else {
+	return eval { $_[0] =~ /$_[1]/i };
+	}
+}
+
 1;
