@@ -1,7 +1,7 @@
 # params-lib.pl
 # Functions for editing parameters common to many kinds of directive
 
-# display_params(&config)
+# display_params(&config, type)
 sub display_params
 {
 print &opt_input($text{'plib_deflt'}, "default-lease-time",
@@ -37,12 +37,59 @@ if ($config{'dhcpd_version'} >= 3) {
 	print &opt_input($text{'plib_ddnsrevdom'}, "ddns-rev-domainname", $_[0], $text{'default'}, 15);
 	print &opt_input($text{'plib_ddnshost'}, "ddns-hostname", $_[0], $text{'plib_ddnshost_def'}, 15);
 	print "</tr>\n";
+
+	if ($_[1] eq 'global') {
+		print "<tr>\n";
+		print &wide_choice_input($text{'plib_ddnsupstyle'}, "ddns-update-style", $_[0], $text{'plib_adhoc'}, 'ad-hoc', $text{'plib_interim'}, 'interim', $text{'plib_none'}, 'none', $text{'default'}, '');
+		print "</tr>\n";
+		}
+
+	# Inputs for allow/deny clients
+	local @adi = ( &find("allow", $_[0]), &find("deny", $_[0]),
+		       &find("ignore", $_[0]) );
+	local ($a, %vals);
+	foreach $a (@adi) {
+		$vals{$a->{'values'}->[0]} = $a;
+		}
+	local $uc = $vals{'unknown-clients'}->{'name'};
+	print "<tr> <td><b>$text{'plib_unclients'}</b></td> <td colspan=3>\n";
+	printf "<input type=radio name=unclients value=allow %s> %s\n",
+		$uc eq 'allow' ? "checked" : "", $text{'plib_allow'};
+	printf "<input type=radio name=unclients value=deny %s> %s\n",
+		$uc eq 'deny' ? "checked" : "", $text{'plib_deny'};
+	printf "<input type=radio name=unclients value=ignore %s> %s\n",
+		$uc eq 'ignore' ? "checked" : "", $text{'plib_ignore'};
+	printf "<input type=radio name=unclients value='' %s> %s\n",
+		$uc ? "" : "checked", $text{'default'};
+	print "</td> </tr>\n";
+
+	if ($_[1] eq 'subnet' || $_[1] eq 'shared-network' ||
+	    $_[1] eq 'global') {
+		# Inputs for authoratative
+		local $auth = &find("authoritative", $_[0]);
+		print "<tr> <td><b>",$text{'plib_auth_'.$_[1]},"</b></td>\n";
+		printf "<td><input type=radio name=auth value=1 %s> %s\n",
+			$auth ? "checked" : "", $text{'yes'};
+		printf "<input type=radio name=auth value=0 %s> %s (%s)</td>\n",
+			$auth ? "" : "checked", $text{'default'}, $text{'no'};
+
+		print "</tr>\n";
+		}
 	}
+
 }
 
 # parse_params(&parent, [&indent])
 sub parse_params
 {
+# Check for expressions
+local $c;
+foreach $c (@{$_[0]->{'members'}}) {
+	if ($c->{'values'}->[0] eq "=") {
+		&error(&text('plib_eexpr', "<tt>$c->{'name'}</tt>"));
+		}
+	}
+
 &save_opt("default-lease-time", \&check_lease, $_[0], $_[1]);
 &save_opt("filename", undef, $_[0], $_[1], 1);
 &save_opt("max-lease-time", \&check_lease, $_[0], $_[1]);
@@ -55,6 +102,31 @@ if ($config{'dhcpd_version'} >= 3) {
 	&save_opt("ddns-rev-domainname", \&check_domain, $_[0], $_[1], 1);
 	&save_opt("ddns-hostname", \&check_server, $_[0], $_[1], 1);
 	&save_choice("ddns-updates", $_[0], $_[1]);
+	if (defined($in{'ddns-update-style'})) {
+		&save_choice("ddns-update-style", $_[0], $_[1]);
+		}
+	local $pm = $_[0]->{'members'};
+	local @adi = ( &find("allow", $pm), &find("deny", $pm),
+		       &find("ignore", $pm) );
+	local ($a, %vals);
+	foreach $a (@adi) {
+		$vals{$a->{'values'}->[0]} = $a;
+		}
+	&save_directive($_[0],
+		$vals{'unknown-clients'} ? [ $vals{'unknown-clients'} ] : [ ],
+		$in{'unclients'} ? [ { 'name' => $in{'unclients'},
+				       'values' => [ 'unknown-clients' ] } ]
+				 : [ ], $_[1]);
+	if (defined($in{'auth'})) {
+		if ($in{'auth'}) {
+			&save_directive($_[0], "authoritative",
+					[ { 'name' => 'authoritative' } ],
+					$_[1]);
+			}
+		else {
+			&save_directive($_[0], "authoritative", [ ], $_[1]);
+			}
+		}
 	}
 }
 
