@@ -3,7 +3,7 @@
 do '../web-lib.pl';
 &init_config();
 &foreign_require("bind8", "bind8-lib.pl");
-do '../bind8/records-lib.pl';
+do "$root_directory/bind8/records-lib.pl";
 
 # mapping between record types and names
 %code_map = ("A", "Address", "NS", "Name Server", "CNAME", "Name Alias",
@@ -210,5 +210,10 @@ foreach (split(/\s+/, $access{'zones'})) {
 return 0;
 }
 
+sub make_chroot
+{
+return $_[0];
+}
+
 1;
 
