@@ -4,223 +4,315 @@
 do '../web-lib.pl';
 &init_config();
 &foreign_require("mount", "mount-lib.pl");
-&foreign_require("raid", "raid-lib.pl") if (&foreign_check("raid"));
+if (&foreign_check("raid")) {
+	&foreign_require("raid", "raid-lib.pl");
+	$raid_module++;
+	}
+if (&foreign_check("lvm")) {
+	&foreign_require("lvm", "lvm-lib.pl");
+	$lvm_module++;
+	}
 &foreign_require("proc", "proc-lib.pl");
 %access = &get_module_acl();
+$has_e2label = &has_command("e2label");
+$has_xfs_db = &has_command("xfs_db");
 $| = 1;
 
-# list_disks()
-# Returns a list of structures, one per disk
-sub list_disks
+# list_disks_partitions()
+# Returns a structure containing the details of all disks and partitions
+sub list_disks_partitions
 {
-return @list_disks_cache if (defined(@list_disks_cache));
-local (@rv, $f, $i);
-local $idx = 0;
-if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
-	# Get the list of IDE disks from the kernel
-	opendir(IDE, "/proc/ide");
-	foreach $f (sort { $a cmp $b } readdir(IDE)) {
-		next if ($f !~ /hd(\S)$/);
-		local $disk = { "device" => "/dev/$f",
-				"prefix" => "/dev/$f",
-				"type" => "ide" };
-		open(MEDIA, "/proc/ide/$f/media");
-		local $media = <MEDIA>;
-		close(MEDIA);
-		next if ($media !~ /^disk/);
-		open(GEOM, "/proc/ide/$f/geometry");
-		while(<GEOM>) {
-			if (/logical\s+(\d+)\/(\d+)\/(\d+)/) {
-				$disk->{'cylinders'} = $1;
-				$disk->{'heads'} = $2;
-				$disk->{'sectors'} = $3;
+if (defined(@list_disks_partitions_cache)) {
+	return @list_disks_partitions_cache;
+	}
+
+# Check /proc/scsi for SCSI disk models
+local @pscsi;
+open(SCSI, "/proc/scsi/scsi");
+while(<SCSI>) {
+	s/\s/ /g;
+	if (/^Host:/) {
+		push(@pscsi, $_);
+		}
+	elsif (/^\s+\S/) {
+		$pscsi[$#pscsi] .= $_;
+		}
+	}
+close(SCSI);
+@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
+
+local (@disks, @devs, $d);
+if (open(PARTS, "/proc/partitions")) {
+	# The list of all disks can come from the kernel
+	local $sc = 0;
+	while(<PARTS>) {
+		if (/\d+\s+\d+\s+\d+\s+sd(\S)\s/ ||
+		    /\d+\s+\d+\s+\d+\s+(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
+			# New or old style SCSI device
+			local $d = $1;
+			local ($host, $bus, $target, $lun) = ($2, $3, $4, $5);
+			if ($pscsi[$sc] =~ /USB-FDU/) {
+				# USB floppy with scsi emulation!
+				splice(@pscsi, $sc, 1);
+				next;
+				}
+			if ($host ne '') {
+				local $scsidev = "/dev/$d";
+				if (!-r $scsidev) {
+					push(@devs, "/dev/sd".chr(97+$sc));
+					}
+				else {
+					push(@devs, $scsidev);
+					}
+				}
+			else {
+				push(@devs, "/dev/sd$d");
+				}
+			$sc++;
+			}
+		elsif (/\d+\s+\d+\s+\d+\s+hd(\S)\s/) {
+			# IDE disk (but skip CDs)
+			local $n = $1;
+			if (open(MEDIA, "/proc/ide/hd$n/media")) {
+				local $media = <MEDIA>;
+				close(MEDIA);
+				if ($media =~ /^disk/) {
+					push(@devs, "/dev/hd$n");
+					}
 				}
 			}
-		close(GEOM);
-		open(MODEL, "/proc/ide/$f/model");
-		chop($disk->{'model'} = <MODEL>);
-		close(MODEL);
-		$disk->{'index'} = $idx++;
-		$disk->{'desc'} = &text('select_device', 'IDE',
-					uc(substr($disk->{'device'}, -1)));
-		push(@rv, $disk);
-		}
-	closedir(IDE);
-
-	# Get the list of SCSI disks from the kernel and fdisk
-	local (@scsi, %dac, %raid, %ida);
-	open(PARTS, "/proc/partitions");
-	while(<PARTS>) {
-		if (/\d+\s+\d+\s+\d+\s+sd(\S)\s/) {
-			# Old-style SCSI disk
-			push(@scsi, "/dev/sd$1");
-			}
-		elsif (/\d+\s+\d+\s+\d+\s+(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
-			# New complete SCSI disk specification
-			push(@scsi, "/dev/$1");
-			$scsidev{"/dev/$1"} = [ $2, $3, $4, $5 ];
+		elsif (/\d+\s+\d+\s+\d+\s+(ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
+			# New-style IDE disk
+			local $idedev = "/dev/$1";
+			local ($host, $bus, $target, $lun) = ($2, $3, $4, $5);
+			if (!-r $idedev) {
+				push(@devs, "/dev/hd".chr(97 + $target +
+							       $bus*2 +
+							       $host*4));
+				}
+			else {
+				push(@devs, "/dev/$1");
+				}
 			}
 		elsif (/\d+\s+\d+\s+\d+\s+(rd\/c(\d+)d\d+)\s/) {
 			# Mylex raid device
-			local ($d, $c) = ("/dev/$1", $2);
-			push(@scsi, $d);
-			open(RD, "/proc/rd/c$c/current_status");
-			while(<RD>) {
-				if (/^Configuring\s+(.*)/i) {
-					$dac{$d} = $1;
-					}
-				elsif (/\s+(\S+):\s+([^, ]+)/ && $1 eq $d) {
-					$raid{$d} = $2;
-					}
-				}
-			close(RD);
+			push(@devs, "/dev/$1");
 			}
 		elsif (/\d+\s+\d+\s+\d+\s+(ida\/c(\d+)d\d+)\s/) {
 			# Compaq raid device
-			local ($d, $c) = ($1, "/dev/$1", $2);
-			push(@scsi, $d);
-			open(IDA, "/proc/driver/array/ida$c");
-			while(<IDA>) {
-				if (/^(\S+):\s+(.*)/ && $1 eq "ida$c") {
-					$ida{$d} = $2;
-					}
-				}
-			close(IDA);
+			push(@devs, "/dev/$1");
+			}
+		elsif (/\d+\s+\d+\s+\d+\s+(cciss\/c(\d+)d\d+)\s/) {
+			# Compaq Smart Array RAID
+			push(@devs, "/dev/$1");
+			}
+		elsif (/\d+\s+\d+\s+\d+\s+(ataraid\/disc(\d+)\/disc)\s+/) {
+			# Promise raid controller
+			push(@devs, "/dev/$1");
 			}
 		}
 	close(PARTS);
-	local @pscsi;
-	open(SCSI, "/proc/scsi/scsi");
-	while(<SCSI>) {
-		s/\s/ /g;
-		if (/^Host:/) {
-			push(@pscsi, $_);
+	@devs = sort { ($b =~ /\/hd[a-z]$/ ? 1 : 0) <=>
+		       ($a =~ /\/hd[a-z]$/ ? 1 : 0) } @devs;
+	}
+
+# Call fdisk to get partition and geometry information
+local @cdstat = stat("/dev/cdrom");
+if (@cdstat) {
+	@devs = grep { (stat($_))[1] != $cdstat[1] } @devs;
+	}
+local $devs = join(" ", @devs);
+local ($disk, $m2);
+open(FDISK, "fdisk -l $devs 2>/dev/null |");
+while(<FDISK>) {
+	if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/ ||
+	    ($m2 = ($_ =~ /Disk\s+([^ :]+):\s+(.*)\s+bytes/))) {
+		# New disk section
+		if ($m2) {
+			$disk = { 'device' => $1,
+				  'prefix' => $1 };
+			<FDISK> =~ /(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/ || next;
+			$disk->{'heads'} = $1;
+			$disk->{'sectors'} = $2;
+			$disk->{'cylinders'} = $3;
 			}
-		elsif (/^\s+\S/) {
-			$pscsi[$#pscsi] .= $_;
+		else {
+			$disk = { 'device' => $1,
+				  'prefix' => $1,
+				  'heads' => $2,
+				  'sectors' => $3,
+				  'cylinders' => $4 };
 			}
-		}
-	close(SCSI);
-	@pscsi = grep { /Type:\s+Direct-Access/i } @pscsi;
-	for($i=0; $i<@scsi; $i++) {
-		next if ($pscsi[$i] =~ /USB-FDU/i);     # skip USB floppies!
-		&open_fdisk("-l", $scsi[$i]);
-		while(<$fh>) {
-			if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/) {
-				local @st = stat($1);
-				local $disk = { 'device' => $1,
-						'heads' => $2,
-						'sectors' => $3,
-						'cylinders' => $4,
-						'index' => $idx++,
-						'type' => 'scsi' };
-				if ($pscsi[$i] =~ /Vendor:\s+(\S+).*Model:\s+(.*)\s+Rev:/i) {
-					$disk->{'model'} = "$1 $2";
+		$disk->{'index'} = scalar(@disks);
+		$disk->{'parts'} = [ ];
+		local @st = stat($disk->{'device'});
+		next if (@cdstat && $st[1] == $cdstat[1]);
+		if ($disk->{'device'} =~ /\/sd(\S)$/) {
+			# Old-style SCSI disk
+			$disk->{'desc'} = &text('select_device', 'SCSI',uc($1));
+			$disk->{'scsi'} = ord(uc($1))-65;
+			$disk->{'type'} = 'scsi';
+			}
+		elsif ($disk->{'device'} =~ /\/hd(\S)$/) {
+			# IDE disk
+			$disk->{'desc'} = &text('select_device', 'IDE', uc($1));
+			$disk->{'type'} = 'ide';
+			}
+		elsif ($disk->{'device'} =~ /\/(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)/) {
+			# New complete SCSI disk specification
+			$disk->{'host'} = $2;
+			$disk->{'bus'} = $3;
+			$disk->{'target'} = $4;
+			$disk->{'lun'} = $5;
+			$disk->{'desc'} = &text('select_scsi',
+						"$2", "$3", "$4", "$5");
+			local $j;
+			for($j=0; $j<@pscsi; $j++) {
+				if ($pscsi[$j] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i && $disk->{'host'} == $1 && $disk->{'target'} == $2) {
+					$disk->{'scsi'} = $j;
+					last;
 					}
-				if ($pscsi[$i] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i) {
-					$disk->{'controller'} = int($1);
-					$disk->{'scsiid'} = int($2);
-					}
-				if ($dac{$disk->{'device'}}) {
-					$disk->{'model'} = $dac{$disk->{'device'}};
-					$disk->{'raid'} = $raid{$disk->{'device'}};
-					$disk->{'device'} =~ /c(\d+)d(\d+)$/;
-					$disk->{'desc'} = &text('select_mylex', "$1", "$2");
-					$disk->{'prefix'} = $disk->{'device'}.'p';
-					}
-				elsif ($ida{$disk->{'device'}}) {
-					$disk->{'model'} = $ida{$disk->{'device'}};
-					$disk->{'device'} =~ /c(\d+)d(\d+)$/;
-					$disk->{'desc'} = &text('select_cpq', "$1", "$2");
-					$disk->{'prefix'} = $disk->{'device'}.'p';
-					}
-				elsif ($scsidev{$disk->{'device'}}) {
-					local @sd = @{$scsidev{$disk->{'device'}}};
-					$disk->{'desc'} = &text('select_scsi', @sd);
-					($disk->{'prefix'} = $disk->{'device'}) =~ s/disc$/part/g;
-					}
-				else {
-					$disk->{'desc'} = &text('select_device', 'SCSI',
-								uc(substr($disk->{'device'}, -1)));
-					$disk->{'prefix'} = $disk->{'device'};
-					}
-				push(@rv, $disk);
 				}
+			$disk->{'type'} = 'scsi';
+			$disk->{'prefix'} =~ s/disc$/part/g;
+			}
+		elsif ($disk->{'device'} =~ /\/(ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)/) {
+			# New-style IDE specification
+			$disk->{'host'} = $2;
+			$disk->{'bus'} = $3;
+			$disk->{'target'} = $4;
+			$disk->{'lun'} = $5;
+			$disk->{'desc'} = &text('select_newide',
+						"$2", "$3", "$4", "$5");
+			$disk->{'type'} = 'ide';
+			$disk->{'prefix'} =~ s/disc$/part/g;
+			}
+		elsif ($disk->{'device'} =~ /\/(rd\/c(\d+)d(\d+))/) {
+			# Mylex raid device
+			local ($mc, $md) = ($2, $3);
+			$disk->{'desc'} = &text('select_mylex', $mc, $md);
+			open(RD, "/proc/rd/c$mc/current_status");
+			while(<RD>) {
+				if (/^Configuring\s+(.*)/i) {
+					$disk->{'model'} = $1;
+					}
+				elsif (/\s+(\S+):\s+([^, ]+)/ &&
+				       $1 eq $disk->{'device'}) {
+					$disk->{'raid'} = $2;
+					}
+				}
+			close(RD);
+			$disk->{'type'} = 'raid';
+			$disk->{'prefix'} = $disk->{'device'}.'p';
+			}
+		elsif ($disk->{'device'} =~ /\/(ida\/c(\d+)d(\d+))/) {
+			# Compaq RAID device
+			local ($ic, $id) = ($2, $3);
+			$disk->{'desc'} = &text('select_cpq', $ic, $id);
+			open(IDA, -d "/proc/driver/array" ? "/proc/driver/array/ida$ic" : "/proc/driver/cpqarray/ida$ic");
+			while(<IDA>) {
+				if (/^(\S+):\s+(.*)/ && $1 eq "ida$ic") {
+					$disk->{'model'} = $2;
+					}
+				}
+			close(IDA);
+			$disk->{'type'} = 'raid';
+			$disk->{'prefix'} = $disk->{'device'}.'p';
+			}
+		elsif ($disk->{'device'} =~ /\/(cciss\/c(\d+)d(\d+))/) {
+			# Compaq Smart Array RAID
+			local ($ic, $id) = ($2, $3);
+			$disk->{'desc'} = &text('select_smart', $ic, $id);
+			open(CCI, "/proc/driver/cciss/cciss$ic");
+			while(<CCI>) {
+				if (/^\s*(\S+):\s*(.*)/ && $1 eq "cciss$ic") {
+					$disk->{'model'} = $2;
+					}
+				}
+			close(CCI);
+			$disk->{'type'} = 'raid';
+			$disk->{'prefix'} = $disk->{'device'}.'p';
+			}
+		elsif ($disk->{'device'} =~ /\/(ataraid\/disc(\d+)\/disc)/) {
+			# Promise RAID controller
+			local $dd = $2;
+			$disk->{'desc'} = &text('select_promise', $dd);
+			$disk->{'type'} = 'raid';
+			$disk->{'prefix'} =~ s/disc$/part/g;
+			}
+		push(@disks, $disk);
+		}
+	elsif (/^Units\s+=\s+cylinders\s+of\s+(\d+)\s+\*\s+(\d+)/) {
+		# Unit size for disk
+		$disk->{'bytes'} = $2;
+		$disk->{'cylsize'} = $disk->{'heads'} * $disk->{'sectors'} *
+				     $disk->{'bytes'};
+		}
+	elsif (/(\/dev\/\S+?(\d+))[ \t*]+\d+\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)/ || /(\/dev\/\S+?(\d+))[ \t*]+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)/) {
+		# Partition within the current disk
+		local $part = { 'number' => $2,
+				'device' => $1,
+				'type' => $6,
+				'start' => $3,
+				'end' => $4,
+				'blocks' => int($5),
+				'extended' => $6 eq '5' || $6 eq 'f' ? 1 : 0,
+				'index' => scalar(@{$disk->{'parts'}}) };
+		$part->{'desc'} =
+			$part->{'device'} =~ /(.)d(\S)(\d+)$/ ?
+			 &text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
+			$part->{'device'} =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
+			 &text('select_spart', "$1", "$2", "$3", "$4", "$5") :
+			$part->{'device'} =~ /ide\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
+			 &text('select_snewide', "$1", "$2", "$3", "$4", "$5") :
+			$part->{'device'} =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ? 
+			 &text('select_mpart', "$1", "$2", "$3") :
+			$part->{'device'} =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ? 
+			 &text('select_cpart', "$1", "$2", "$3") :
+			$part->{'device'} =~ /cciss\/c(\d+)d(\d+)p(\d+)$/ ? 
+			 &text('select_smartpart', "$1", "$2", "$3") :
+			$part->{'device'} =~ /ataraid\/disc(\d+)\/part(\d+)$/ ?
+			 &text('select_ppart', "$1", "$2") :
+			"???",
+		push(@{$disk->{'parts'}}, $part);
+		}
+	}
+close(FDISK);
+
+# Check /proc/ide for IDE disk models
+foreach $d (@disks) {
+	if ($d->{'type'} eq 'ide') {
+		local $short;
+		if (defined($d->{'host'})) {
+			$short = "hd".(('a' .. 'z')[$d->{'host'}*4 + $d->{'target'}*2 + $d->{'bus'}]);
+			}
+		else {
+			$short = $d->{'device'};
+			$short =~ s/^.*\///g;
+			}
+		if (open(MODEL, "/proc/ide/$short/model")) {
+			($d->{'model'} = <MODEL>) =~ s/\r|\n//g;
+			close(MODEL);
 			}
-		&close_fdisk();
 		}
 	}
 
-if (!@rv) {
-	# Get the list from fdisk -l or sfdisk -l
-	local @cdstat = stat("/dev/cdrom");
-	if (&has_command("sfdisk")) {
-		&open_sfdisk("-l");
-	} else {
-		&open_fdisk("-l");
-	}
-	while(<$fh>) {
-		if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/) {
-			local @st = stat($1);
-			if ($st[1] != $cdstat[1]) {
-				local $disk = { 'device' => $1,
-						'prefix' => $1,
-						'heads' => $2,
-						'sectors' => $3,
-						'cylinders' => $4,
-						'index' => $idx++ };
-				$disk->{'type'} =
-				    $disk->{'device'} =~ /hd.$/ ? 'ide'
-								: 'scsi';
-				$disk->{'desc'} = &text('select_device', uc($disk->{'type'}),
-							uc(substr($disk->{'device'}, -1)));
-				push(@rv, $disk);
-				}
+# Fill in SCSI information
+foreach $d (@disks) {
+	if ($d->{'type'} eq 'scsi') {
+		local $s = $d->{'scsi'};
+		if ($pscsi[$s] =~ /Vendor:\s+(\S+).*Model:\s+(.*)\s+Rev:/i) {
+			$d->{'model'} = "$1 $2";
+			}
+		if ($pscsi[$s] =~ /Host:\s+scsi(\d+).*Id:\s+(\d+)/i) {
+			$d->{'controller'} = int($1);
+			$d->{'scsiid'} = int($2);
 			}
 		}
-	&close_fdisk();
 	}
-@list_disks_cache = @rv;
-return @rv;
-}
 
-# list_partitions(disk)
-# Returns a list of structures, one per partition
-sub list_partitions
-{
-return @{$list_partitions_cache{$_[0]}}
-	if (defined($list_partitions_cache{$_[0]}));
-local(@rv, $wf);
-local $idx = 0;
-&open_fdisk($_[0]);
-&wprint("p\nq\n");
-&wait_for($fh, 'System\r');
-while(1) {
-	$wf = &wait_for($fh,
-	  '\n(\/dev\/\S+?(\d+))[ \t*]+\d+\s+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)\r',
-	  '\n(\/dev\/\S+?(\d+))[ \t*]+(\d+)\s+(\d+)\s+(\S+)\s+(\S{1,2})\s+(.*)\r');
-	last if ($wf < 0);
-	$matches[4] =~ s/\+$//g;
-	push(@rv, { 'number' => $matches[2],
-		    'device' => $matches[1],
-		    'type' => $matches[6],
-		    'start' => $matches[3],
-		    'end' => $matches[4],
-		    'blocks' => $matches[5],
-		    'extended' => $matches[6] eq '5' || $matches[6] eq 'f' ? 1 : 0,
-		    'desc' => $matches[1] =~ /(.)d(\S)(\d+)$/ ?
-			      &text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
-			      $matches[1] =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
-			      &text('select_spart', "$1", "$2", "$3", "$4", "$5") :
-			      $matches[1] =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ? 
-			      &text('select_mpart', "$1", "$2", "$3") :
-			      $matches[1] =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ? 
-			      &text('select_cpart', "$1", "$2", "$3") : "???",
-		    'index' => $idx++ });
-	}
-&close_fdisk();
-$list_partitions_cache{$_[0]} = \@rv;
-return @rv;
+@list_disks_partitions_cache = @disks;
+return @disks;
 }
 
 # change_type(disk, partition, type)
@@ -229,8 +321,8 @@ sub change_type
 {
 &open_fdisk("$_[0]");
 &wprint("t\n");
-&wait_for($fh, 'Partition.*:');
-&wprint("$_[1]\n");
+local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
+&wprint("$_[1]\n") if ($rv == 0);
 &wait_for($fh, 'Hex.*:');
 &wprint("$_[2]\n");
 &wait_for($fh, 'Command.*:');
@@ -244,8 +336,8 @@ sub delete_partition
 {
 &open_fdisk("$_[0]");
 &wprint("d\n");
-&wait_for($fh, 'Partition.*:');
-&wprint("$_[1]\n");
+local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
+&wprint("$_[1]\n") if ($rv == 0);
 &wait_for($fh, 'Command.*:');
 &wprint("w\n");
 &wait_for($fh, 'Syncing'); sleep(3);
@@ -264,8 +356,8 @@ if ($_[1] > 4) {
 	}
 else {
 	&wprint("p\n");
-	&wait_for($fh, 'Partition.*:');
-	&wprint("$_[1]\n");
+	local $wf2 = &wait_for($fh, 'Partition.*:', 'Selected partition');
+	&wprint("$_[1]\n") if ($wf2 == 0);
 	}
 &wait_for($fh, 'First.*:') if ($wf != 1);
 &wprint("$_[2]\n");
@@ -274,8 +366,8 @@ else {
 &wait_for($fh, 'Command.*:');
 
 &wprint("t\n");
-&wait_for($fh, 'Partition.*:');
-&wprint("$_[1]\n");
+local $rv = &wait_for($fh, 'Partition.*:', 'Selected partition');
+&wprint("$_[1]\n") if ($rv == 0);
 &wait_for($fh, 'Hex.*:');
 &wprint("$_[4]\n");
 &wait_for($fh, 'Command.*:');
@@ -337,7 +429,7 @@ return undef;
 # Returns a readable name for a filesystem type
 sub fstype_name
 {
-return $fstypes{$_[0]};
+return $text{"fs_".$_[0]};
 }
 
 sub mkfs_options
@@ -406,6 +498,13 @@ elsif ($_[0] eq "xfs") {
 
 	&opt_input("xfs_b", $text{'bytes'}, 0);
 	}
+elsif ($_[0] eq "jfs") {
+	&opt_input("jfs_s", $text{'megabytes'}, 1);
+	print "<td align=right><b>$text{'jfs_c'}</b></td>\n";
+	print "<td><input type=radio name=jfs_c value=1> $text{'yes'}\n";
+	print "<input type=radio name=jfs_c value=0 checked> $text{'no'}",
+	      "</td> </tr>\n";
+	}
 }
 
 # mkfs_parse(type, device)
@@ -453,15 +552,25 @@ elsif ($_[0] eq "reiserfs") {
 elsif ($_[0] eq "ext3") {
 	if (&has_command("mkfs.ext3")) {
 		$cmd = "mkfs -t ext3";
+		$cmd .= &opt_check("ext3_j", '\d+', "-j");
+		}
+	elsif (&has_command("mke3fs")) {
+		$cmd = "mke3fs";
+		$cmd .= &opt_check("ext3_j", '\d+', "-j");
 		}
 	else {
-		$cmd = "mke3fs";
+		$cmd = "mkfs.ext2 -j";
+		if (!$in{'ext3_j_def'}) {
+			$in{'ext3_j'} =~ /^\d+$/ ||
+				&error(&text('opt_error', $in{'ext3_j'},
+					     $text{'ext3_j'}));
+			$cmd .= " -J size=$in{'ext3_j'}";
+			}
 		}
 	$cmd .= &opt_check("ext2_b", '\d+', "-b");
 	$cmd .= &opt_check("ext2_f", '\d+', "-f");
 	$cmd .= &opt_check("ext2_i", '\d{4,}', "-i");
 	$cmd .= &opt_check("ext2_m", '\d+', "-m");
-	$cmd .= &opt_check("ext3_j", '\d+', "-j");
 	$cmd .= $in{'ext2_c'} ? " -c" : "";
 	$cmd .= " $_[1]";
 	}
@@ -471,6 +580,12 @@ elsif ($_[0] eq "xfs") {
 	$cmd .= " -b size=$in{'xfs_b'}" if (!$in{'xfs_b_def'});
 	$cmd .= " $_[1]";
 	}
+elsif ($_[0] eq "jfs") {
+	$cmd = "mkfs -t $_[0] -q";
+	$cmd .= &opt_check("jfs_s", '\d+', "-s");
+	$cmd .= " -c" if ($in{'jfs_c'});
+	$cmd .= " $_[1]";
+	}
 return $cmd;
 }
 
@@ -554,26 +669,39 @@ sub device_status
 {
 @mounted = &foreign_call("mount", "list_mounted") if (!@mounted);
 @mounts = &foreign_call("mount", "list_mounts") if (!@mounts);
-local $label = `e2label $_[0] 2>/dev/null`;
-chop($label);
+local $label = &get_label($_[0]);
 
-local ($mounted) = grep { $_->[1] eq $_[0] ||
+local ($mounted) = grep { &same_file($_->[1], $_[0]) ||
 			  $_->[1] eq "LABEL=$label" } @mounted;
-local ($mount) = grep { $_->[1] eq $_[0] ||
+local ($mount) = grep { &same_file($_->[1], $_[0]) ||
 			$_->[1] eq "LABEL=$label" } @mounts;
 if ($mounted) { return ($mounted->[0], $mounted->[2], 1,
 			&indexof($mount, @mounts),
 			&indexof($mounted, @mounted)); }
 elsif ($mount) { return ($mount->[0], $mount->[2], 0,
 			 &indexof($mount, @mounts)); }
-elsif (&foreign_check("raid")) {
+if ($raid_module) {
 	$raidconf = &foreign_call("raid", "get_raidtab") if (!$raidconf);
 	foreach $c (@$raidconf) {
-		if (&indexof($_[0], @{$c->{'devices'}}) >= 0) {
-			return ( $c->{'value'}, "raid", 1 );
+		foreach $d (&raid::find_value('device', $c->{'members'})) {
+			return ( $c->{'value'}, "raid", 1 ) if ($d eq $_[0]);
 			}
 		}
 	}
+if ($lvm_module) {
+	if (!defined(@physical_volumes)) {
+		@physical_volumes = ();
+		foreach $vg (&foreign_call("lvm", "list_volume_groups")) {
+			push(@physical_volumes,
+				&foreign_call("lvm", "list_physical_volumes",
+						     $vg->{'name'}));
+			}
+		}
+	foreach $pv (@physical_volumes) {
+		return ( $pv->{'vg'}, "lvm", 1)
+			if ($pv->{'device'} eq $_[0]);
+		}
+	}
 return ();
 }
 
@@ -624,7 +752,7 @@ if ($_[2] == 0 || $_[2] == 2) {
 		&text('select_fd', 1) if (!$_[4] || "/dev/fd1" =~ /$_[4]/);
 	$found++ if ($_[1] =~ /^\/dev\/fd[01]$/);
 	}
-local @dlist = &list_disks();
+local @dlist = &list_disks_partitions();
 foreach $d (@dlist) {
 	local $dev = $d->{'device'};
 	next if ($_[4] && $dev !~ /$_[4]/);
@@ -636,8 +764,7 @@ foreach $d (@dlist) {
 		$found++ if ($dev eq $_[1]);
 		}
 	if ($_[2] == 0 || $_[2] == 2 || $_[2] == 3) {
-		local @parts = &list_partitions($dev);
-		foreach $p (@parts) {
+		foreach $p (@{$d->{'parts'}}) {
 			next if ($p->{'extended'});
 			local $name = $p->{'desc'};
 			$name .= " (".&tag_name($p->{'type'}).")"
@@ -663,16 +790,14 @@ return $rv;
 sub label_select
 {
 local $rv = "<select name=$_[0]>\n";
-local @dlist = &list_disks();
+local @dlist = &list_disks_partitions();
 local $any;
 foreach $d (@dlist) {
 	local $dev = $d->{'device'};
-	local @parts = &list_partitions($dev);
-	foreach $p (@parts) {
+	foreach $p (@{$d->{'parts'}}) {
 		next if ($p->{'type'} ne '83');
-		local $label = `e2label $p->{'device'} 2>&1`;
-		next if ($? || $label !~ /\S/);
-		chop($label);
+		local $label = &get_label($p->{'device'});
+		next if (!$label);
 		$rv .= sprintf "<option %s value=%s>%s (%s)\n",
 			  $_[1] eq $label ? "selected" : "",
 			  $label, $label, $p->{'desc'};
@@ -693,13 +818,13 @@ return $any ? $rv : undef;
 sub open_fdisk
 {
 local $fpath = &check_fdisk();
-($fh, $fpid) = &foreign_call("proc", "pty_process_exec", $fpath, @_);
+($fh, $fpid) = &foreign_call("proc", "pty_process_exec", join(" ",$fpath, @_));
 }
 
 sub open_sfdisk
 {
 local $sfpath = &has_command("sfdisk");
-($fh, $fpid) = &foreign_call("proc", "pty_process_exec", $sfpath, @_);
+($fh, $fpid) = &foreign_call("proc", "pty_process_exec", join(" ",$sfpath, @_));
 }
 
 sub check_fdisk
@@ -823,14 +948,6 @@ else { return " $_[2] $in{$_[0]}"; }
 	 'f', 'Win95 extended LBA',
 	);
 	
-%fstypes = ("ext2","Linux Native",
-	    "minix","Minix",
-	    "msdos","MS-DOS",
-	    "vfat","Windows 95",
-	    "reiserfs","ReiserFS",
-	    "ext3","New Linux Native",
-	    "xfs","SGI");
-
 @space_type = ( '1', '4', '5', '6', 'b', 'c', 'e', '83' );
 
 # can_edit_disk(device)
@@ -842,12 +959,70 @@ foreach (split(/\s+/, $access{'disks'})) {
 return 0;
 }
 
-# disk_space(device)
+# disk_space(device, [mountpoint])
 # Returns the amount of total and free space for some filesystem, or an
 # empty array if not appropriate.
 sub disk_space
 {
-`df -k $_[0]` =~ /Mounted on\n\S+\s+(\S+)\s+\S+\s+(\S+)/ || return ();
-return ($1, $2);
+local $w = $_[1] || $_[0];
+local $out = `df -k '$w'`;
+if ($out =~ /Mounted on\s*\n\s*\S+\s+(\S+)\s+\S+\s+(\S+)/i) {
+	return ($1, $2);
+	}
+elsif ($out =~ /Mounted on\s*\n\S+\s*\n\s+(\S+)\s+\S+\s+(\S+)/i) {
+	return ($1, $2);
+	}
+else {
+	return ( );
+	}
 }
 
+# supported_filesystems()
+# Returns a list of filesystem types that can have mkfs_options called on them
+sub supported_filesystems
+{
+local @fstypes = ( "ext2" );
+push(@fstypes, "ext3") if (&has_command("mkfs.ext3") ||
+			   &has_command("mke3fs") ||
+			   `mkfs.ext2 -h 2>&1` =~ /\[-j\]/);
+push(@fstypes, "reiserfs") if (&has_command("mkreiserfs"));
+push(@fstypes, "xfs") if (&has_command("mkfs.xfs"));
+push(@fstypes, "jfs") if (&has_command("mkfs.jfs"));
+push(@fstypes, "msdos");
+push(@fstypes, "vfat");
+push(@fstypes, "minix");
+return @fstypes;
+}
+
+# get_label(device, [type])
+# Returns the XFS or EXT label for some device's filesystem
+sub get_label
+{
+local $label;
+if ($has_e2label) {
+	$label = `e2label $_[0] 2>&1`;
+	chop($label);
+	}
+if (($? || $label !~ /\S/) && $has_xfs_db) {
+	$label = undef;
+	local $out = `xfs_db -x -p xfs_admin -c label -r $_[0] 2>&1`;
+	$label = $1 if ($out =~ /label\s*=\s*"(.*)"/ &&
+			$1 ne '(null)');
+	}
+return $? || $label !~ /\S/ ? undef : $label;
+}
+
+# set_label(device, label, [type])
+# Tries to set the label for some device's filesystem
+sub set_label
+{
+if ($has_e2label && ($_[2] =~ /^ext[23]$/ || !$_[2])) {
+	&system_logged("e2label '$_[0]' '$_[1]' >/dev/null 2>&1");
+	return 1 if (!$?);
+	}
+if ($has_xfs_db && ($_[2] eq "xfs" || !$_[2])) {
+	&system_logged("xfs_db -x -p xfs_admin -c \"label $_[1]\" $_[0] >/dev/null 2>&1");
+	return 1 if (!$?);
+	}
+return 0;
+}
