@@ -1,4 +1,5 @@
 import java.awt.*;
+import java.util.*;
 
 class ErrorWindow extends FixedFrame implements CbButtonCallback
 {
@@ -8,8 +9,11 @@ class ErrorWindow extends FixedFrame implements CbButtonCallback
 	{
 	setLayout(new BorderLayout());
 	Panel cen = new BorderPanel(1);
-	cen.setLayout(new GridLayout(1, 1));
-	cen.add(new Label(m));
+	StringTokenizer tok = new StringTokenizer(m, "\r\n");
+	cen.setLayout(new GridLayout(tok.countTokens(), 1));
+	while(tok.hasMoreTokens()) {
+		cen.add(new Label(tok.nextToken()));
+		}
 	add("Center", cen);
 	Panel bot = new GrayPanel();
 	bot.setLayout(new FlowLayout(FlowLayout.CENTER));
@@ -27,3 +31,4 @@ class ErrorWindow extends FixedFrame implements CbButtonCallback
 	}
 }
 
+
