@@ -13,6 +13,7 @@ public class Hierarchy extends BorderPanel implements CbScrollbarCallback
 	int sbwidth;			// size of scrollbar
 	HierarchyCallback callback;	// who to call on open / close
 	Image bim;			// double-buffer image
+	Font font = new Font("courier", Font.PLAIN, 12);
 	FontMetrics fnm;		// size of font used
 	Graphics bg;			// back-images graphics
 	int top = 0;			// top-most row displayed
@@ -85,12 +86,20 @@ public class Hierarchy extends BorderPanel implements CbScrollbarCallback
 	}
 
 	// select
-	// Send the given node
+	// Selected the given node
 	void select(HierarchyNode s)
 	{
 	sel = s;
 	}
 
+	// force the use of some font
+	public void setFont(Font f)
+	{
+	font = f;
+	bim = null;
+	repaint();
+	}
+
 	// reshape
 	// Called when this component gets resized
 	public void reshape(int nx, int ny, int nw, int nh)
@@ -126,7 +135,7 @@ public class Hierarchy extends BorderPanel implements CbScrollbarCallback
 		// This is the first rendering
 		bim = createImage(width, height);
 		bg = bim.getGraphics();
-		bg.setFont(new Font("courier", Font.PLAIN, 12));
+		bg.setFont(font);
 		fnm = bg.getFontMetrics();
 		render();
 		compscroll();
