@@ -100,7 +100,7 @@ foreach $f (@files) {
 	open(INET, $f);
 	while(<INET>) {
 		chop;
-		if (/^(#+|#<off>#)?\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\/\S+|internal)\s*(.*)$/) {
+		if (/^(#+|#<off>#)?\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\??\/\S+|internal)\s*(.*)$/) {
 			push(@rv, [ $l, !$1, 0, $2, $3, $4,
 				    $5, $6, $7, $8, $f ]);
 			$rv[$#rv]->[2] = ($2 =~ /\//);
