@@ -1,5 +1,5 @@
 
-require './init-lib.pl';
+require 'init-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the init module
