@@ -253,7 +253,12 @@ local $conf = &get_config();
 local $ldir = &perl_var_replace(&find_value("listdir", $conf), $conf);
 local @ldir = stat($ldir);
 chown($ldir[4], $ldir[5], $_[0]);
-chmod(-d $_[0] ? 0775 : 0664, $_[0]);
+if ($config{'perms'}) {
+	chmod(-d $_[0] ? 0755 : 0644, $_[0]);
+	}
+else {
+	chmod(-d $_[0] ? 0775 : 0664, $_[0]);
+	}
 }
 
 # choice_input(name, text, &config, [opt, display]+)
@@ -342,5 +347,21 @@ foreach (split(/\s+/, $_[0]->{'lists'})) {
 return 0;
 }
 
+# homedir_valid(&config)
+sub homedir_valid
+{
+local $homedir = &find_value("homedir", $_[0]);
+if (!-d $homedir) {
+	local $homeused;
+	foreach $c (@$conf) {
+		$homeused++ if ($c->{'value'} =~ /\$homedir/);
+		}
+	if ($homeused) {
+		return 0;
+		}
+	}
+return 1;
+}
+
 1;
 
