@@ -1,5 +1,5 @@
 
-do './majordomo-lib.pl';
+do 'majordomo-lib.pl';
 $conf = &get_config();
 @lists = &list_lists($conf);
 
@@ -53,6 +53,27 @@ foreach $l (@lists) {
 # Does nothing
 sub useradmin_modify_user
 {
+return if ($_[0]->{'user'} eq $_[0]->{'olduser'} || !$_[0]->{'olduser'});
+foreach $l (@lists) {
+	if ($config{"sync_$l"}) {
+		# Directly update the subscription list, if the user is on it
+		local $dom = $config{"shost_$l"};
+		$dom = &get_system_hostname() if (!$dom);
+		local $list = &get_list($l, $conf);
+		&lock_file($list->{'members'});
+		local $lref = &read_file_lines($list->{'members'});
+		local ($i, $found);
+		for($i=0; $i<@$lref; $i++) {
+			if ($lref->[$i] eq "$_[0]->{'olduser'}\@$dom") {
+				$lref->[$i] = "$_[0]->{'user'}\@$dom";
+				$found++;
+				}
+			}
+		&flush_file_lines() if ($found);
+		&unlock_file($list->{'members'});
+		}
+	}
+
 }
 
 1;
