@@ -71,6 +71,17 @@ if ($_[1] ne "-") {
 	}
 }
 
+# join_options(type, [&hash])
+# Returns a string constructed from the %options hash
+sub join_options
+{
+local $h = $_[1] || \%options;
+local (@rv, $k);
+foreach $k (keys %$h) {
+	push(@rv, $h->{$k} eq "" ? $k : $k."=".$h->{$k});
+	}
+return @rv ? join(",", @rv) : "-";
+}
 
 # swap_form(path)
 # This function should be called by os-specific code to display a form
@@ -102,7 +113,7 @@ sub nfs_server_chooser_button
 {
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
-print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield' value=\"...\">\n";
+print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$module_name/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 }
 
 # nfs_export_chooser_button(serverinput, exportinput, [form])
@@ -110,7 +121,7 @@ sub nfs_export_chooser_button
 {
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
-print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; }' value=\"...\">\n";
+print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$module_name/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
 }
 
 # smb_server_chooser_button(serverinput, [form])
@@ -119,7 +130,7 @@ sub smb_server_chooser_button
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
 if (&has_command($config{'smbclient_path'})) {
-	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield' value=\"...\">\n";
+	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"../$module_name/smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 	}
 }
 
@@ -129,16 +140,54 @@ sub smb_share_chooser_button
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
 if (&has_command($config{'smbclient_path'})) {
-	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; }' value=\"...\">\n";
+	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"../$module_name/smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
 	}
 }
 
 # Include the correct OS-specific functions file
-$os_lib_file = "./$gconfig{os_type}-lib.pl";
-if (-r $os_lib_file) {
-	do $os_lib_file;
+do "$gconfig{'os_type'}-lib.pl";
+
+# can_edit_fs(dir, device, type, options)
+# Returns 1 if a filesystem can be edited, 0 otherwise
+sub can_edit_fs
+{
+return 1 if (!$access{'fs'});
+local @dirs = split(/\s+/, $access{'fs'});
+local $d;
+foreach $d (@dirs) {
+	return 1 if ($d eq "/" || $d eq $_[0] ||
+		     substr($_[0], 0, length($d)+1) eq "$d/");
 	}
-else { die "This OS is not supported!"; }
+return 0;
+}
+
+# compile_program(name, default-arch)
+# Ensures that some C program is compiled and copied to /etc/webmin . Uses the
+# supplied native versions if possible
+sub compile_program
+{
+return if (-r "$module_config_directory/$_[0]" &&
+	   system("$module_config_directory/$_[0] >/dev/null 2>&1 </dev/null")
+		== 0);
+local $arch = `uname -m`;
+$arch =~ s/\r|\n//g;
+local $re = $_[1];
+if ($re && $arch =~ /^$re$/i && -r "$module_root_directory/$_[0]" &&
+    system("$module_root_directory/$_[0] >/dev/null 2>&1 </dev/null") == 0) {
+	# Compiled program for this architecture already exists and is working,
+	# so can just copy
+	system("cp $module_root_directory/$_[0] $module_config_directory/$_[0]");
+	}
+else {
+	# Need to compile
+	local ($cc) = (&has_command("gcc") || &has_command("cc"));
+	$cc || &error($text{'egcc'});
+	local $out = `$cc -o $module_config_directory/$_[0] $module_root_directory/$_[0].c 2>&1`;
+	if ($?) {
+		&error(&text('ecompile', "<pre>$out</pre>"));
+		}
+	}
+}
 
 1;
 
