@@ -1,6 +1,8 @@
 # openbsd-lib.pl
 # Mount table functions for openbsd
 
+$uname_release = `uname -r`;
+
 # Return information about a filesystem, in the form:
 #  directory, device, type, options, fsck_order, mount_at_boot
 # If a field is unused or ignored, a - appears instead of the value.
@@ -124,7 +126,10 @@ sub list_mounted
 {
 # get the list of mounted filesystems
 local(@rv, $_);
-open(CMD, "./openbsd-mounts-2 |");
+local $cmd = $uname_release =~ /^3\./ ? "openbsd-mounts-3" :
+					"openbsd-mounts-2";
+&compile_program($cmd, '.*86');
+open(CMD, "$module_config_directory/$cmd |");
 while(<CMD>) {
 	local @p = split(/\t/, $_);
 	if ($p[2] eq "procfs" || $p[1] eq "procfs") { $p[1] = $p[2] = "proc"; }
