@@ -9,75 +9,14 @@ use File::Copy;
 
 $network_interfaces_config = '/etc/network/interfaces';
 
-# active_interfaces()
-# Returns a list of currently ifconfig'd interfaces
-sub active_interfaces
-{
-local(@rv, @lines, $l);
-open(IFC, "ifconfig -a |");
-while(<IFC>) {
-	s/\r|\n//g;
-	if (/^\S+/) { push(@lines, $_); }
-	else { $lines[$#lines] .= $_; }
-	}
-close(IFC);
-foreach $l (@lines) {
-	local %ifc;
-	$l =~ /^([^:\s]+)/; $ifc{'name'} = $1;
-	$l =~ /^(\S+)/; $ifc{'fullname'} = $1;
-	if ($l =~ /^(\S+):(\d+)/) { $ifc{'virtual'} = $2; }
-	if ($l =~ /inet addr:(\S+)/) { $ifc{'address'} = $1; }
-	else { next; }
-	if ($l =~ /Mask:(\S+)/) { $ifc{'netmask'} = $1; }
-	if ($l =~ /Bcast:(\S+)/) { $ifc{'broadcast'} = $1; }
-	if ($l =~ /HWaddr (\S+)/) { $ifc{'ether'} = $1; }
-	if ($l =~ /MTU:(\d+)/) { $ifc{'mtu'} = $1; }
-	$ifc{'up'}++ if ($l =~ /\sUP\s/);
-	$ifc{'edit'} = ($ifc{'name'} !~ /^ppp/);
-	$ifc{'index'} = scalar(@rv);
-	push(@rv, \%ifc);
-	}
-return @rv;
-}
-
-# activate_interface(&details)
-# Create or modify an interface
-sub activate_interface
-{
-local $a = $_[0];
-local $cmd = "ifconfig $a->{'name'}";
-if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
-$cmd .= " $a->{'address'}";
-if ($a->{'netmask'}) { $cmd .= " netmask $a->{'netmask'}"; }
-if ($a->{'broadcast'}) { $cmd .= " broadcast $a->{'broadcast'}"; }
-if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
-if ($a->{'up'}) { $cmd .= " up"; }
-else { $cmd .= " down"; }
-local $out = &backquote_logged("$cmd 2>&1");
-if ($?) { &error($out); }
-if ($a->{'ether'}) {
-	$out = &backquote_logged(
-		"ifconfig $a->{'name'} hw ether $a->{'ether'} 2>&1");
-	if ($?) { &error($out); }
-	}
-}
-
-# deactivate_interface(&details)
-# Shutdown some active interface
-sub deactivate_interface
-{
-local $cmd = "ifconfig $a->{'name'}";
-if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
-$cmd .= " down";
-local $out = &backquote_logged("$cmd 2>&1");
-if ($?) { &error($out); }
-}
+do 'linux-lib.pl';
 
 # boot_interfaces()
 # Returns a list of interfaces brought up at boot time
 sub boot_interfaces()
 {
 my @ifaces = &get_interface_defs();
+my @autos = &get_auto_defs();
 my @rv;
 foreach $iface (@ifaces) {
 	my ($name, $addrfam, $method, $options) = @$iface;
@@ -91,7 +30,12 @@ foreach $iface (@ifaces) {
 		else {
 			$cfg->{'name'} = $cfg->{'fullname'};
 			}
-		$cfg->{'up'} = 1;
+		if ($gconfig{'os_version'} >= 3 || scalar(@autos)) {
+			$cfg->{'up'} = &indexof($name, @autos) >= 0;
+			}
+		else {
+			$cfg->{'up'} = 1;
+			}
 		foreach $option (@$options) {
 			($param, $value) = @$option;
 			if ($param eq 'noauto') { $cfg->{'up'} = 0; }
@@ -134,7 +78,9 @@ else {
 		push(@options, ['network', $network]);
 		}
 	}
-if (! $cfg->{'up'}) { push(@options, ['noauto', '']); }
+my @autos = get_auto_defs();
+my $amode = $gconfig{'os_version'} > 3 || scalar(@autos);
+if (!$cfg->{'up'} && !$amode) { push(@options, ['noauto', '']); }
 
 my @ifaces = get_interface_defs();
 my $changeit = 0;
@@ -154,6 +100,15 @@ if ($changeit == 0) {
 else {
 	modify_interface_def($cfg->{'fullname'}, 'inet', $method, \@options, 0);
 	}
+if ($amode) {
+	if ($cfg->{'up'}) {
+		@autos = &unique(@autos, $cfg->{'fullname'});
+		}
+	else {
+		@autos = grep { $_ ne $cfg->{'fullname'} } @autos;
+		}
+	&modify_auto_defs(@autos);
+	}
 }
 
 # delete_interface(&details)
@@ -162,30 +117,11 @@ sub delete_interface
 {
 my $cfg = $_[0];
 delete_interface_def($cfg->{'fullname'}, 'inet');
-}
-
-# iface_type(name)
-# Returns a human-readable interface type name
-sub iface_type
-{
-return "PPP" if ($_[0] =~ /^ppp/);
-return "SLIP" if ($_[0] =~ /^sl/);
-return "PLIP" if ($_[0] =~ /^plip/);
-return "Ethernet" if ($_[0] =~ /^eth/);
-return "Arcnet" if ($_[0] =~ /^arc/);
-return "Token Ring" if ($_[0] =~ /^tr/);
-return "Pocket/ATP" if ($_[0] =~ /^atp/);
-return "Loopback" if ($_[0] =~ /^lo/);
-return "ISDN rawIP" if ($_[0] =~ /^isdn/);
-return "ISDN syncPPP" if ($_[0] =~ /^ippp/);
-return $text{'ifcs_unknown'};
-}
-
-# iface_hardware(name)
-# Does some interface have an editable hardware address
-sub iface_hardware
-{
-return $_[0] =~ /^eth/;
+my @autos = get_auto_defs();
+if ($gconfig{'os_version'} > 3 || scalar(@autos)) {
+	@autos = grep { $_ ne $cfg->{'fullname'} } @autos;
+	&modify_auto_defs(@autos);
+	}
 }
 
 # can_edit(what)
@@ -326,19 +262,23 @@ else {
 # get_hostname()
 sub get_hostname
 {
-return &get_system_hostname();
+return &get_system_hostname(1);
 }
 
 # save_hostname(name)
 sub save_hostname
 {
-local %conf;
+local (%conf, $f);
 &system_logged("hostname $_[0] >/dev/null 2>&1");
-&lock_file("/etc/HOSTNAME");
-open(HOST, ">/etc/HOSTNAME");
-print HOST $_[0],"\n";
-close(HOST);
-&unlock_file("/etc/HOSTNAME");
+foreach $f ("/etc/hostname", "/etc/HOSTNAME") {
+	if (-r $f) {
+		&lock_file($f);
+		open(HOST, ">$f");
+		print HOST $_[0],"\n";
+		close(HOST);
+		&unlock_file($f);
+		}
+	}
 &lock_file($network_config);
 &read_env_file($network_config, \%conf);
 $conf{'HOSTNAME'} = $_[0];
@@ -372,53 +312,35 @@ else {
 # show default router and device
 sub routing_input
 {
+local ($addr, $router) = &get_default_gateway();
 local @ifaces = &get_interface_defs();
-local ($router, $addr);
-foreach $iface (@ifaces) {
-	foreach $o (@{$iface->[3]}) {
-		if ($o->[0] eq 'gateway') {
-			$router = $iface;
-			$addr = $o->[1];
-			}
-		}
-	}
 
 print "<tr> <td><b>$text{'routes_default'}</b></td>\n";
 printf "<td><input type=radio name=gateway_def value=1 %s> %s\n",
-	$router ? '' : 'checked', $text{'routes_none'};
-printf "<input type=radio name=gateway_def value=0 %s>\n",
-	$router ? 'checked' : '';
-print "$text{'routes_gateway'}\n";
+	$addr ? '' : 'checked', $text{'routes_none'};
+printf "<input type=radio name=gateway_def value=0 %s> %s\n",
+	$addr ? 'checked' : '', $text{'routes_gateway'};
 printf "<input name=gateway size=15 value='%s'>\n", $addr;
 print "$text{'routes_device'}\n";
 print "<select name=gatewaydev>\n";
 foreach $iface (@ifaces) {
 	next if ($iface->[0] eq 'lo');
 	printf "<option %s>%s\n",
-		$router eq $iface ? 'selected' : '', $iface->[0];
+		$router eq $iface->[0] ? 'selected' : '', $iface->[0];
 	}
 print "</select></td> </tr>\n";
 }
 
 sub parse_routing
 {
-local $dev;
+local ($dev, $gw);
 if (!$in{'gateway_def'}) {
 	&check_ipaddress($in{'gateway'}) ||
 		&error(&text('routes_egateway', $in{'gateway'}));
+	$gw = $in{'gateway'};
 	$dev = $in{'gatewaydev'};
 	}
-local @ifaces = &get_interface_defs();
-foreach $iface (@ifaces) {
-	# Remove the gateway directive
-	$iface->[3] = [ grep { $_->[0] ne 'gateway' } @{$iface->[3]} ];
-
-	# Add if needed
-	if ($iface->[0] eq $dev) {
-		push(@{$iface->[3]}, [ 'gateway', $in{'gateway'} ]);
-		}
-	&modify_interface_def(@$iface);
-	}
+&set_default_gateway($gw, $dev);
 }
 
 
@@ -465,7 +387,7 @@ while (defined $line) {
 			next;
 			}
 		}
-	elsif (my ($name, $addrfam, $method) = ($line =~ /^\s*iface\s+(\S+)\s+(\w+)\s+(\w+)\s*$/) ) {
+	elsif (my ($name, $addrfam, $method) = ($line =~ /^\s*iface\s+(\S+)\s+(\S+)\s+(\S+)\s*$/) ) {
 		# only lines starting with "iface" are expected here
 		my @iface_options;
 		# now read everything until the next iface definition
@@ -498,6 +420,49 @@ close(CFGFILE);
 return @ret;
 }
 
+# get_auto_defs()
+# Returns a list of interfaces in auto lines
+sub get_auto_defs
+{
+local @rv;
+open(CFGFILE, $network_interfaces_config);
+while(<CFGFILE>) {
+	s/\r|\n//g;
+	s/^\s*#.*$//g;
+	if (/^\s*auto\s*(.*)/) {
+		push(@rv, split(/\s+/, $1));
+		}
+	}
+close(CFGFILE);
+return @rv;
+}
+
+# modify_auto_defs(iface, ...)
+# Replaces all auto lines with one containing the interfaces given as params
+sub modify_auto_defs
+{
+local $lref = &read_file_lines($network_interfaces_config);
+for($i=0; $i<@$lref; $i++) {
+	local $l = $lref->[$i];
+	$l =~ s/\r|\n//g;
+	$l =~ s/^\s*#.*$//g;
+	if ($l =~ /^\s*auto\s*(.*)/) {
+		if (!$found++) {
+			# Replace the auto line
+			$lref->[$i] = "auto ".join(" ", @_);
+			}
+		else {
+			# Remove another auto line
+			splice(@$lref, $i--, 1);
+			}
+		}
+	}
+if (!$found) {
+	splice(@$lref, 0, 0, "auto ".join(" ", @_));
+	}
+&flush_file_lines();
+}
+
 # modifies the options of an already stored interface definition
 # the parameters should be (name, addrfam, method, options, mode)
 # with options being an array of (param, value) pairs
@@ -519,13 +484,13 @@ my $iface_line = 0;
 my $new_options_wrote;
 while (defined ($line=<OLDCFGFILE>)) {
 	if ($inside_modify_region == 0 &&
-		$line =~ /^\s*iface\s+$name\s+$addrfam\s+\w+\s*$/) {
+           $line =~ /^\s*iface\s+$name\s+$addrfam\s+\S+\s*$/) {
 		$inside_modify_region = 1;
 		$iface_line = 1;
 		$new_options_wrote = 0;
 		}
 	elsif ($inside_modify_region == 1 &&
-		$line =~ /^\s*iface\s+\w+\s+\w+\s+\w+\s*$/) {
+              $line =~ /^\s*iface\s+\S+\s+\S+\s+\S+\s*$/) {
 		$inside_modify_region = 0;
 		}
 	# preserve comments
@@ -534,23 +499,21 @@ while (defined ($line=<OLDCFGFILE>)) {
 		}
 	# inside modify region or not ?
 	elsif ($inside_modify_region == 0) {
-    	print NEWCFGFILE $line;
+               print NEWCFGFILE $line;
 		}
 	else {
 		# should the iface line be changed or the options ?
 		if ($iface_line == 1 && $mode == 0) {
 			print NEWCFGFILE "iface $name $addrfam $method\n";
 			}
-		else {
-			# only write the new options and skip the old ones or just do
-			# nothing if mode is delete
-			# write only upon first entrance here
-			if ($mode == 0 && $new_options_wrote == 0) {
-				$new_options_wrote = 1;
-				foreach $option (@$options) {
-					my ($param, $value) = @$option;
-					print NEWCFGFILE "\t$param $value\n";
-					}
+               # only write the new options and skip the old ones or just do
+               # nothing if mode is delete
+               # write only upon first entrance here
+               if ($mode == 0 && $new_options_wrote == 0) {
+                       $new_options_wrote = 1;
+                       foreach $option (@$options) {
+                               my ($param, $value) = @$option;
+                               print NEWCFGFILE "\t$param $value\n";
 				}
 			}
 		}
@@ -598,5 +561,48 @@ return ( $network_interfaces_config, "/etc/nsswitch.conf", "/etc/resolv.conf",
 	 "/etc/HOSTNAME" );
 }
 
+# apply_network()
+# Apply the interface and routing settings
+sub apply_network
+{
+&system_logged("(cd / ; /etc/init.d/networking stop ; /etc/init.d/networking start) >/dev/null 2>&1");
+}
+
+# get_default_gateway()
+# Returns the default gateway IP (if one is set) and device (if set) boot time
+# settings.
+sub get_default_gateway
+{
+local @ifaces = &get_interface_defs();
+local ($router, $addr);
+foreach $iface (@ifaces) {
+	foreach $o (@{$iface->[3]}) {
+		if ($o->[0] eq 'gateway') {
+			return ( $o->[1], $iface->[0] );
+			}
+		}
+	}
+return ( );
+}
+
+# set_default_gateway([gateway, device])
+# Sets the default gateway to the given IP accessible via the given device,
+# in the boot time settings.
+sub set_default_gateway
+{
+local @ifaces = &get_interface_defs();
+local $iface;
+foreach $iface (@ifaces) {
+	# Remove the gateway directive
+	$iface->[3] = [ grep { $_->[0] ne 'gateway' } @{$iface->[3]} ];
+
+	# Add if needed
+	if ($iface->[0] eq $_[1]) {
+		push(@{$iface->[3]}, [ 'gateway', $_[0] ]);
+		}
+	&modify_interface_def(@$iface);
+	}
+}
+
 1;
 
