@@ -1,6 +1,8 @@
 # solaris-lib.pl
 # Networking functions for solaris
 
+$min_virtual_number = 1;
+
 # active_interfaces()
 # Returns a list of currently ifconfig'd interfaces
 sub active_interfaces
@@ -70,6 +72,7 @@ if ($a->{'ether'}) {
 # Deactive an interface
 sub deactivate_interface
 {
+local $a = $_[0];
 local $cmd;
 if ($a->{'virtual'} eq "") {
 	$cmd = "ifconfig $a->{'name'} unplumb";
@@ -119,15 +122,19 @@ while($f = readdir(ETC)) {
 		if ($ifc{'address'}) {
 			&to_ipaddress($ifc{'address'})
 				=~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
-			if ($mask{"$1.$2.$3.0"})
-				{ $ifc{'netmask'} = $mask{"$1.$2.$3.0"}; }
-			elsif ($mask{"$1.$2.0.0"})
-				{ $ifc{'netmask'} = $mask{"$1.$2.0.0"}; }
-			elsif ($mask{"$1.0.0.0"})
-				{ $ifc{'netmask'} = $mask{"$1.0.0.0"}; }
-			else
-				{ $ifc{'netmask'} = "255.255.255.0"; }
 			local ($a1, $a2, $a3, $a4) = ($1, $2, $3, $4);
+			$ifc{'netmask'} = "255.255.255.0";
+			local $netaddr;
+			foreach $netaddr (keys %mask)
+			{
+				$mask{$netaddr} =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
+				local $na = sprintf "%d.%d.%d.%d",
+						int($a1) & int($1),
+						int($a2) & int($2),
+						int($a3) & int($3),
+						int($a4) & int($4);
+				$ifc{'netmask'} = $mask{$netaddr}  if ($na eq $netaddr);
+			}
 			$ifc{'netmask'} =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/;
 			$ifc{'broadcast'} = sprintf "%d.%d.%d.%d",
 						($a1 | ~int($1))&0xff,
