@@ -48,5 +48,28 @@ foreach $ia (keys %text) {
 
 $has_fuser_command = 0;
 
+# get_new_pty()
+# Returns the filehandles and names for a pty and tty
+sub get_new_pty
+{
+local @ptys;
+opendir(DEV, "/dev");
+@ptys = map { "/dev/$_" } (grep { /^pty/ } readdir(DEV));
+closedir(DEV);
+local ($pty, $tty);
+foreach $pty (@ptys) {
+	open(PTY, "+>$pty") || next;
+	local $tty = $pty;
+	$tty =~ s/pty/tty/;
+	open(TTY, "+>$tty") || next;
+	local $old = select(PTY); $| = 1;
+	select(TTY); $| = 1; select($old);
+	return (*PTY, *TTY, $pty, $tty);
+	}
+return ();
+}
+
+
+
 1;
 
