@@ -16,6 +16,8 @@ $qmail_local_dir = "$config{'qmail_dir'}/queue/local";
 $qmail_remote_dir = "$config{'qmail_dir'}/queue/remote";
 $qmail_assigns_file = "$config{'qmail_dir'}/users/assign";
 
+$config{'perpage'} ||= 20;      # a value of 0 can cause problems
+
 # list_aliases()
 # Returns a list of qmail alias file names
 sub list_aliases
@@ -477,6 +479,7 @@ print "</table></td></tr></table></form>\n";
 sub list_queue
 {
 local (@rv, %qmap);
+@rv = ( );
 opendir(DIR, $qmail_mess_dir);
 foreach $m (readdir(DIR)) {
 	next if ($m !~ /^\d+$/);
@@ -507,37 +510,6 @@ close(QUEUE);
 return @rv;
 }
 
-# read_mail_file(file)
-sub read_mail_file
-{
-local (@headers, $mail);
-
-# Read the headers
-open(MAIL, $_[0]);
-while(1) {
-	local $line = <MAIL>;
-	$line =~ s/\r|\n//g;
-	last if ($line eq '');
-	if ($line =~ /^(\S+):\s*(.*)/) {
-		push(@headers, [ $1, $2 ]);
-		}
-	elsif ($line =~ /^(\s+.*)/) {
-		$headers[$#headers]->[1] .= $1 unless($#headers < 0);
-		}
-	}
-$mail->{'headers'} = \@headers;
-foreach $h (@headers) {
-	$mail->{'header'}->{lc($h->[0])} = $h->[1];
-	}
-
-# Read the mail body
-while(read(MAIL, $buf, 1024) > 0) {
-	$mail->{'body'} .= $buf;
-	}
-close(MAIL);
-return $mail;
-}
-
 # wrap_lines(text, width)
 # Given a multi-line string, return an array of lines wrapped to
 # the given width
@@ -689,7 +661,30 @@ print "</td> </tr>\n";
 print "</table></td></tr></table></form>\n";
 }
 
+# user_mail_dir(username, ...)
+# Returns the full path to a user's mail file or directory
+sub user_mail_dir
+{
+if ($config{'mail_system'} == 1) {
+	if (@_ > 1) {
+		return "$_[7]/$config{'mail_dir_qmail'}/";
+		}
+	else {
+		local @u = getpwnam($_[0]);
+		return "$u[7]/$config{'mail_dir_qmail'}/";
+		}
+	}
+else {
+	return &user_mail_file(@_);
+	}
+}
 
+# restart_qmail()
+# Tells qmail to reload its configuration files by sending a HUP signal
+sub restart_qmail
+{
+&kill_byname_logged("qmail-send", HUP);
+}
 
 1;
 
