@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './sendmail-lib.pl';
+do 'sendmail-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -13,10 +13,14 @@ if ($type eq 'alias' || $type eq 'virtuser' || $type eq 'mailer' ||
 	return &text("log_${type}_${action}",
 		     "<tt>".&html_escape($object)."</tt>");
 	}
+elsif ($type eq 'feature') {
+	return &text("log_feature_${action}",
+		     "<tt>".&html_escape($p->{'text'})."</tt>");
+	}
 elsif ($action eq 'delmailq') {
 	if ($p->{'from'}) {
 		return &text("log_delmailq",
-			     &html_escape(extract_email($p->{'from'})));
+			     &html_escape(&extract_email($p->{'from'})));
 		}
 	else {
 		return &text("log_delmailqs", $p->{'count'});
@@ -26,6 +30,12 @@ elsif ($action eq 'delmail') {
 	local @d = split(/\0/, $p->{'d'});
 	return &text("log_delmail", scalar(@d), "<tt>$p->{'user'}</tt>");
 	}
+elsif ($action eq 'movemail') {
+	local @d = split(/\0/, $p->{'d'});
+	local $to = $p->{'move1'} ? $p->{'moveto1'} : $p->{'moveto2'};
+	return &text("log_movemail", scalar(@d), "<tt>$p->{'user'}</tt>",
+		     "<tt>$to</tt>");
+	}
 elsif ($action eq 'send') {
 	return &text('log_send', &html_escape(&extract_email($p->{'to'})));
 	}
@@ -40,10 +50,10 @@ else {
 sub extract_email
 {
 if ($_[0] =~ /([^<>"' \(\)]+\@[^<>"' \(\)]+)/) {
-	return "<tt>$1</tt>";
+	return $1;
 	}
 elsif ($_[0] =~ /<(\S+)>/) {
-	return "<tt>$1</tt>";
+	return $1;
 	}
 else {
 	return $_[0];
