@@ -4,16 +4,30 @@
 # to setup the various config files
 
 # Find install directory
+LANG=
+export LANG
 cd `dirname $0`
 if [ -x /bin/pwd ]; then
 	wadir=`/bin/pwd`
 else
 	wadir=`pwd`;
 fi
-ver=`cat $wadir/version`
+srcdir=$wadir
+ver=`cat "$wadir/version"`
+
+# Find temp directory
+if [ "$tempdir" = "" ]; then
+	tempdir=/tmp/.webmin
+fi
+
+if [ $? != "0" ]; then
+	echo "ERROR: Cannot find the Webmin install directory";
+	echo "";
+	exit 1;
+fi
 
 echo "***********************************************************************"
-echo "*            Welcome to the Webmin setup script, version $ver         *"
+echo "*            Welcome to the Webmin setup script, version $ver        *"
 echo "***********************************************************************"
 echo "Webmin is a web-based interface that allows Unix-like operating"
 echo "systems and common Unix services to be easily administered."
@@ -30,9 +44,33 @@ if [ $? != "0" ]; then
 	fi
 fi
 
-# Display install directory
-echo "Installing Webmin in $wadir ..."
-allmods=`cd $wadir; ls */module.info | sed -e 's/\/module.info//g' | xargs echo`
+# Use the supplied destination directory, if any
+if [ "$1" != "" ]; then
+	wadir=$1
+	echo "Installing Webmin from $srcdir to $wadir ..."
+	if [ ! -d "$wadir" ]; then
+		mkdir "$wadir"
+		if [ "$?" != "0" ]; then
+			echo "ERROR: Failed to create $wadir"
+			echo ""
+			exit 1
+		fi
+	else
+		# Make sure dest dir is not in use
+		ls "$wadir" | grep -v rpmsave >/dev/null 2>&1
+		if [ "$?" = "0" -a ! -r "$wadir/setup.sh" ]; then
+			echo "ERROR: Installation directory $wadir contains other files"
+			echo ""
+			exit 1
+		fi
+	fi
+else
+	echo "Installing Webmin in $wadir ..."
+fi
+cd "$wadir"
+
+# Validate source directory
+allmods=`cd "$srcdir"; echo */module.info | sed -e 's/\/module.info//g'`
 if [ "$allmods" = "" ]; then
 	echo "ERROR: Failed to get module list"
 	echo ""
@@ -40,6 +78,11 @@ if [ "$allmods" = "" ]; then
 fi
 echo ""
 
+# Load package-defined variable overrides
+if [ -r "$srcdir/setup-pre.sh" ]; then
+	. "$srcdir/setup-pre.sh"
+fi
+
 # Ask for webmin config directory
 echo "***********************************************************************"
 echo "Webmin uses separate directories for configuration files and log files."
@@ -75,17 +118,24 @@ fi
 
 # Check if upgrading from an old version
 if [ "$upgrading" = 1 ]; then
+	echo ""
+
 	# Get current var path
 	var_dir=`cat $config_dir/var-path`
 
+	# Force creation if non-existant
+	mkdir -p $var_dir >/dev/null 2>&1
+
 	# Get current perl path
 	perl=`cat $config_dir/perl-path`
 
-	# Stop old version
-	$config_dir/stop >/dev/null 2>&1
-
-	# Update ACLs
-	$perl $wadir/newmods.pl $config_dir $allmods
+	# Create temp files directory
+	$perl "$srcdir/maketemp.pl"
+	if [ "$?" != "0" ]; then
+		echo "ERROR: Failed to create or check temp files directory $tempdir"
+		echo ""
+		exit 2
+	fi
 
 	# Get old os name and version
 	os_type=`grep "^os_type=" $config_dir/config | sed -e 's/os_type=//g'`
@@ -95,14 +145,30 @@ if [ "$upgrading" = 1 ]; then
 
 	# Get old root, host, port, ssl and boot flag
 	oldwadir=`grep "^root=" $config_dir/miniserv.conf | sed -e 's/root=//g'`
-	host=`grep "^host=" $config_dir/miniserv.conf | sed -e 's/host=//g'`
 	port=`grep "^port=" $config_dir/miniserv.conf | sed -e 's/port=//g'`
 	ssl=`grep "^ssl=" $config_dir/miniserv.conf | sed -e 's/ssl=//g'`
 	atboot=`grep "^atboot=" $config_dir/miniserv.conf | sed -e 's/atboot=//g'`
+	inetd=`grep "^inetd=" $config_dir/miniserv.conf | sed -e 's/inetd=//g'`
+
+	if [ "$inetd" != "1" ]; then
+		# Stop old version
+		$config_dir/stop >/dev/null 2>&1
+	fi
+
+	# Copy files to target directory
+	if [ "$wadir" != "$srcdir" ]; then
+		echo "Copying files to $wadir .."
+		(cd "$srcdir" ; tar cf - . | (cd "$wadir" ; tar xf -))
+		echo "..done"
+		echo ""
+	fi
+
+	# Update ACLs
+	$perl "$wadir/newmods.pl" $config_dir $allmods
 
 	# Update miniserv.conf with new root directory and mime types file
-	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" >/tmp/$$.miniserv.conf
-	mv /tmp/$$.miniserv.conf $config_dir/miniserv.conf
+	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" >$tempdir/$$.miniserv.conf
+	mv $tempdir/$$.miniserv.conf $config_dir/miniserv.conf
 	echo "root=$wadir" >> $config_dir/miniserv.conf
 	echo "mimetypes=$wadir/mime.types" >> $config_dir/miniserv.conf
 	grep logout= $config_dir/miniserv.conf >/dev/null
@@ -116,10 +182,13 @@ if [ "$upgrading" = 1 ]; then
 		if [ "$webmin_upgrade" != "" ]; then
 			autothird=1
 		fi
-		$perl $wadir/thirdparty.pl $wadir $oldwadir $autothird
+		$perl "$wadir/thirdparty.pl" "$wadir" "$oldwadir" $autothird
 		echo "..done"
 		echo ""
 	fi
+
+	# Remove old cache of module infos
+	rm -f $config_dir/module.infos.cache
 else
 	# Config directory exists .. make sure it is not in use
 	ls $config_dir | grep -v rpmsave >/dev/null 2>&1
@@ -227,15 +296,26 @@ else
 	echo "Perl seems to be installed ok"
 	echo ""
 
+	# Create temp files directory
+	$perl "$srcdir/maketemp.pl"
+	if [ "$?" != "0" ]; then
+		echo "ERROR: Failed to create or check temp files directory $tempdir"
+		echo ""
+		exit 2
+	fi
+
 	# Ask for operating system type
 	echo "***********************************************************************"
 	if [ "$os_type" = "" ]; then
-		$perl $wadir/oschooser.pl $wadir/os_list.txt /tmp/$$.os $autoos
+		if [ "$autoos" = "" ]; then
+			autoos=2
+		fi
+		$perl "$srcdir/oschooser.pl" "$srcdir/os_list.txt" $tempdir/$$.os $autoos
 		if [ $? != 0 ]; then
 			exit $?
 		fi
-		. /tmp/$$.os
-		rm -f /tmp/$$.os
+		. $tempdir/$$.os
+		rm -f $tempdir/$$.os
 	fi
 	echo "Operating system name:    $real_os_type"
 	echo "Operating system version: $real_os_version"
@@ -249,7 +329,6 @@ else
 	echo "   web server already using this port."
 	echo " - The login name required to access the web server."
 	echo " - The password required to access the web server."
-	echo " - The hostname of this system that the web server should use."
 	echo " - If the webserver should use SSL (if your system supports it)."
 	echo " - Whether to start webmin at boot time."
 	echo ""
@@ -271,7 +350,7 @@ else
 		echo ""
 		exit 12
 	fi
-	$perl -e 'use Socket; socket(FOO, PF_INET, SOCK_STREAM, getprotobyname("tcp")); setsockopt(FOO, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)); bind(FOO, sockaddr_in($ARGV[0], INADDR_ANY)) || exit(1); exit(0);' $port
+	$perl -e 'use Socket; socket(FOO, PF_INET, SOCK_STREAM, getprotobyname("tcp")); setsockopt(FOO, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)); bind(FOO, pack_sockaddr_in($ARGV[0], INADDR_ANY)) || exit(1); exit(0);' $port
 	if [ $? != "0" ]; then
 		echo "ERROR: TCP port $port is already in use by another program"
 		echo ""
@@ -284,12 +363,23 @@ else
 			login="admin"
 		fi
 	fi
-	echo $login | grep : >/dev/null
+	echo "$login" | grep : >/dev/null
 	if [ "$?" = "0" ]; then
 		echo "ERROR: Username contains a : character"
 		echo ""
 		exit 14
 	fi
+	echo $login | grep " " >/dev/null
+	if [ "$?" = "0" ]; then
+		echo "ERROR: Username contains a space"
+		echo ""
+		exit 14
+	fi
+	if [ "$login" = "webmin" ]; then
+		echo "ERROR: Username 'webmin' is reserved for internal use"
+		echo ""
+		exit 14
+	fi
 	printf "Login password: "
 	if [ "$password" = "" -a "$crypt" = "" ]; then
 		stty -echo
@@ -313,14 +403,6 @@ else
 			exit 14
 		fi
 	fi
-	defhost=`hostname`
-	printf "Web server host name (default $defhost): "
-	if [ "$host" = "" ]; then
-		read host
-		if [ "$host" = "" ]; then
-			host=$defhost
-		fi
-	fi
 	if [ "$ssl" = "" ]; then
 		ssl=0
 		$perl -e 'use Net::SSLeay' >/dev/null 2>/dev/null
@@ -338,7 +420,7 @@ else
 
 	# Ask whether to run at boot time
 	if [ "$atboot" = "" ]; then
-		initsupp=`grep "^os_support=" $wadir/init/module.info | sed -e 's/os_support=//g' | grep $os_type`
+		initsupp=`grep "^os_support=" "$srcdir/init/module.info" | sed -e 's/os_support=//g' | grep $os_type`
 		atboot=0
 		if [ "$initsupp" != "" ]; then
 			printf "Start Webmin at boot time (y/n): "
@@ -352,19 +434,27 @@ else
 	fi
 	makeboot=$atboot
 
+	# Copy files to target directory
+	echo "***********************************************************************"
+	if [ "$wadir" != "$srcdir" ]; then
+		echo "Copying files to $wadir .."
+		(cd "$srcdir" ; tar cf - . | (cd "$wadir" ; tar xf -))
+		echo "..done"
+		echo ""
+	fi
+
 	# Create webserver config file
 	echo $perl > $config_dir/perl-path
 	echo $var_dir > $config_dir/var-path
-	echo "***********************************************************************"
 	echo "Creating web server config files.."
 	cfile=$config_dir/miniserv.conf
 	echo "port=$port" >> $cfile
 	echo "root=$wadir" >> $cfile
-	echo "host=$host" >> $cfile
 	echo "mimetypes=$wadir/mime.types" >> $cfile
 	echo "addtype_cgi=internal/cgi" >> $cfile
 	echo "realm=Webmin Server" >> $cfile
 	echo "logfile=$var_dir/miniserv.log" >> $cfile
+	echo "errorlog=$var_dir/miniserv.error" >> $cfile
 	echo "pidfile=$var_dir/miniserv.pid" >> $cfile
 	echo "logtime=168" >> $cfile
 	echo "ppath=$ppath" >> $cfile
@@ -375,6 +465,10 @@ else
 	echo "logout=$config_dir/logout-flag" >> $cfile
 	echo "listen=10000" >> $cfile
 	echo "denyfile=\\.pl\$" >> $cfile
+	echo "log=1" >> $cfile
+	echo "blockhost_failures=5" >> $cfile
+	echo "blockhost_time=60" >> $cfile
+	echo "syslog=1" >> $cfile
 	if [ "$allow" != "" ]; then
 		echo "allow=$allow" >> $cfile
 	fi
@@ -394,7 +488,28 @@ else
 	echo "userfile=$ufile" >> $cfile
 
 	kfile=$config_dir/miniserv.pem
-	cp $wadir/miniserv.pem $kfile
+	openssl version >/dev/null 2>&1
+	if [ "$?" = "0" ]; then
+		# We can generate a new SSL key for this host
+		host=`hostname`
+		openssl req -newkey rsa:512 -x509 -nodes -out $tempdir/cert -keyout $tempdir/key -days 1825 >/dev/null 2>&1 <<EOF
+.
+.
+.
+Webmin Webserver on $host
+.
+*
+root@$host
+EOF
+		if [ "$?" = "0" ]; then
+			cat $tempdir/cert $tempdir/key >$kfile
+		fi
+		rm -f $tempdir/cert $tempdir/key
+	fi
+	if [ ! -r $kfile ]; then
+		# Fall back to the built-in key
+		cp "$wadir/miniserv.pem" $kfile
+	fi
 	chmod 600 $kfile
 	echo "keyfile=$config_dir/miniserv.pem" >> $cfile
 
@@ -405,35 +520,55 @@ else
 	echo "Creating access control file.."
 	afile=$config_dir/webmin.acl
 	rm -f $afile
-	echo "$login: $allmods" >> $afile
+	if [ "$defaultmods" = "" ]; then
+		echo "$login: $allmods" >> $afile
+	else
+		echo "$login: $defaultmods" >> $afile
+	fi
 	chmod 600 $afile
 	echo "..done"
 	echo ""
 
+	if [ "$login" != "root" -a "$login" != "admin" ]; then
+		# Allow use of RPC by this user
+		echo rpc=1 >>$config_dir/$login.acl
+	fi
 fi
 
 if [ "$noperlpath" = "" ]; then
 	echo "Inserting path to perl into scripts.."
-	(find $wadir -name '*.cgi' -print ; find $wadir -name '*.pl' -print) | $perl $wadir/perlpath.pl $perl -
+	(find "$wadir" -name '*.cgi' -print ; find "$wadir" -name '*.pl' -print) | $perl "$wadir/perlpath.pl" $perl -
 	echo "..done"
 	echo ""
 fi
 
 echo "Creating start and stop scripts.."
-rm -f $config_dir/stop $config_dir/start
+rm -f $config_dir/stop $config_dir/start $config_dir/restart
 echo "#!/bin/sh" >>$config_dir/start
 echo "echo Starting Webmin server in $wadir" >>$config_dir/start
 echo "trap '' 1" >>$config_dir/start
+echo "LANG=" >>$config_dir/start
+echo "export LANG" >>$config_dir/start
+echo "#PERLIO=:raw" >>$config_dir/start
+echo "unset PERLIO" >>$config_dir/start
+echo "export PERLIO" >>$config_dir/start
+echo "PERLLIB=$wadir" >>$config_dir/start
+echo "export PERLLIB" >>$config_dir/start
 uname -a | grep -i 'HP/*UX' >/dev/null
 if [ $? = "0" ]; then
-	echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf &" >>$config_dir/start
+	echo "exec '$wadir/miniserv.pl' $config_dir/miniserv.conf &" >>$config_dir/start
 else
-	echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf" >>$config_dir/start
+	echo "exec '$wadir/miniserv.pl' $config_dir/miniserv.conf" >>$config_dir/start
 fi
 
 echo "#!/bin/sh" >>$config_dir/stop
 echo "echo Stopping Webmin server in $wadir" >>$config_dir/stop
-echo "kill \`cat $var_dir/miniserv.pid\`" >>$config_dir/stop
+echo "pidfile=\`grep \"^pidfile=\" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'\`" >>$config_dir/stop
+echo "kill \`cat \$pidfile\`" >>$config_dir/stop
+
+echo "#!/bin/sh" >>$config_dir/restart
+echo "$config_dir/stop && $config_dir/start" >>$config_dir/restart
+
 chmod 755 $config_dir/start $config_dir/stop
 echo "..done"
 echo ""
@@ -443,7 +578,7 @@ if [ "$upgrading" = 1 ]; then
 else
 	echo "Copying config files.."
 fi
-$perl $wadir/copyconfig.pl $os_type $os_version $wadir $config_dir "" $allmods
+$perl "$wadir/copyconfig.pl" "$os_type" "$os_version" "$wadir" $config_dir "" $allmods
 if [ "$upgrading" != 1 ]; then
 	# Store the OS and version
 	echo "os_type=$os_type" >> $config_dir/config
@@ -458,11 +593,14 @@ if [ "$upgrading" != 1 ]; then
 		elif [ "$CONF_LST_LANG" = "uk" ]; then
 			CONF_LST_LANG=en
 		fi
-		grep "lang=$CONF_LST_LANG," $wadir/lang_list.txt >/dev/null 2>&1
+		grep "lang=$CONF_LST_LANG," "$wadir/lang_list.txt" >/dev/null 2>&1
 		if [ "$?" = 0 ]; then
 			echo "lang=$CONF_LST_LANG" >> $config_dir/config
 		fi
 	fi
+
+	# Turn on logging by default
+	echo "log=1" >> $config_dir/config
 else
 	# one-off hack to set log variable in config from miniserv.conf
 	grep log= $config_dir/config >/dev/null
@@ -476,20 +614,38 @@ echo $ver > $config_dir/version
 echo "..done"
 echo ""
 
+# Set passwd_ fields in miniserv.conf from global config
+for field in passwd_file passwd_uindex passwd_pindex passwd_cindex passwd_mindex; do
+	grep $field= $config_dir/miniserv.conf >/dev/null
+	if [ "$?" != "0" ]; then
+		grep $field= $config_dir/config >> $config_dir/miniserv.conf
+	fi
+done
+grep passwd_mode= $config_dir/miniserv.conf >/dev/null
+if [ "$?" != "0" ]; then
+	echo passwd_mode=0 >> $config_dir/miniserv.conf
+fi
+
+# Set a special theme if none was set before
 if [ "$theme" = "" ]; then
-	theme=`cat $wadir/defaulttheme 2>/dev/null`
+	theme=`cat "$wadir/defaulttheme" 2>/dev/null`
 fi
 oldthemeline=`grep "^theme=" $config_dir/config`
 oldtheme=`echo $oldthemeline | sed -e 's/theme=//g'`
-if [ "$theme" != "" ] && [ "$oldthemeline" = "" ] && [ -d $wadir/$theme ]; then
-	# Set a special theme if none was set before
+if [ "$theme" != "" ] && [ "$oldthemeline" = "" ] && [ -d "$wadir/$theme" ]; then
 	echo "theme=$theme" >> $config_dir/config
 	echo "preroot=$theme" >> $config_dir/miniserv.conf
 fi
 
+# Set the product field in the global config
+grep product= $config_dir/config >/dev/null
+if [ "$?" != "0" ]; then
+	echo product=webmin >> $config_dir/config
+fi
+
 if [ "$makeboot" = "1" ]; then
 	echo "Configuring Webmin to start at boot time.."
-	(cd $wadir/init ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir $wadir/init/atboot.pl)
+	(cd "$wadir/init" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/init/atboot.pl")
 	echo "..done"
 	echo ""
 fi
@@ -503,13 +659,12 @@ read answer
 printf "\n"
 if [ "\$answer" = "y" ]; then
 	$config_dir/stop
-	if [ "$atboot" = "1" ]; then
-		(cd $wadir/init ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir $wadir/init/delboot.pl)
-	fi
+	echo "Running uninstall scripts .."
+	(cd "$wadir" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir LANG= "$wadir/run-uninstalls.pl")
 	echo "Deleting $wadir .."
-	rm -rf $wadir
+	rm -rf "$wadir"
 	echo "Deleting $config_dir .."
-	rm -rf $config_dir
+	rm -rf "$config_dir"
 	echo "Done!"
 fi
 EOF
@@ -519,36 +674,59 @@ EOF
 fi
 
 echo "Changing ownership and permissions .."
-chown -R root:bin $config_dir
+chown -R root $config_dir
+chgrp -R bin $config_dir
 chmod -R og-rw $config_dir
-chmod 755 $config_dir/sendmail/config >/dev/null 2>&1
-chmod 755 $config_dir/sendmail/autoreply.pl >/dev/null 2>&1
+chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/config >/dev/null 2>&1
+chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/autoreply.pl >/dev/null 2>&1
+chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/filter.pl >/dev/null 2>&1
+chmod 755 $config_dir/squid*/squid-auth.pl >/dev/null 2>&1
+chmod 755 $config_dir/squid*/users >/dev/null 2>&1
+chmod +r $config_dir/version
 if [ "$nochown" = "" ]; then
-	chown -R root:bin $wadir
-	#chmod -R og-rwx $wadir
+	chown -R root "$wadir"
+	chgrp -R bin "$wadir"
+	chmod -R og-w "$wadir"
+	chmod -R a+rx "$wadir"
 fi
 if [ $var_dir != "/var" ]; then
-	chown -R root:bin $var_dir
+	chown -R root $var_dir
+	chgrp -R bin $var_dir
 	chmod -R og-rwx $var_dir
 fi
 echo "..done"
 echo ""
 
+# Save target directory if one was specified
+if [ "$wadir" != "$srcdir" ]; then
+	echo $wadir >$config_dir/install-dir
+else
+	rm -f $config_dir/install-dir
+fi
+
+echo "Running postinstall scripts .."
+(cd "$wadir" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/run-postinstalls.pl")
+echo "..done"
+echo ""
+
 if [ "$nostart" = "" ]; then
-	echo "Attempting to start Webmin mini web server.."
-	$config_dir/start
-	if [ $? != "0" ]; then
-		echo "ERROR: Failed to start web server!"
+	if [ "$inetd" != "1" ]; then
+		echo "Attempting to start Webmin mini web server.."
+		$config_dir/start
+		if [ $? != "0" ]; then
+			echo "ERROR: Failed to start web server!"
+			echo ""
+			exit 14
+		fi
+		echo "..done"
 		echo ""
-		exit 14
 	fi
-	echo "..done"
-	echo ""
 
 	echo "***********************************************************************"
 	echo "Webmin has been installed and started successfully. Use your web"
 	echo "browser to go to"
 	echo ""
+	host=`hostname`
 	if [ "$ssl" = "1" ]; then
 		echo "  https://$host:$port/"
 	else
@@ -567,3 +745,17 @@ if [ "$nostart" = "" ]; then
 	fi
 fi
 
+if [ "$oldwadir" != "$wadir" -a "$upgrading" = 1 -a "$deletedold" != 1 ]; then
+	echo "The directory from the previous version of Webmin"
+	echo "   $oldwadir"
+	echo "Can now be safely deleted to free up disk space, assuming"
+	echo "that all third-party modules have been copied to the new"
+	echo "version."
+	echo ""
+fi
+
+# Run package-defined post-install script
+if [ -r "$srcdir/setup-post.sh" ]; then
+	. "$srcdir/setup-post.sh"
+fi
+
