@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+%access = &get_module_acl();
 
 # get_config()
 sub get_config
@@ -46,6 +47,9 @@ while($line = <CONF>) {
 			$log->{'file'} = $1;
 			$log->{'sync'} = 1;
 			}
+		elsif ($act =~ /^\@\@(\S+)$/) {
+			$log->{'socket'} = $1;
+			}
 		elsif ($act =~ /^\@(\S+)$/) {
 			$log->{'host'} = $1;
 			}
@@ -123,7 +127,7 @@ sub log_line
 {
 local $d;
 if ($_[0]->{'file'}) {
-	$d = ($_[0]->{'sync'} ? "" : "-").$_[0]->{'file'};
+	$d = ($_[0]->{'sync'} || !$config{'sync'} ? "" : "-").$_[0]->{'file'};
 	}
 elsif ($_[0]->{'pipe'}) {
 	$d = '|'.$_[0]->{'pipe'};
@@ -134,6 +138,9 @@ elsif ($_[0]->{'host'}) {
 elsif ($_[0]->{'users'}) {
 	$d = join(",", @{$_[0]->{'users'}});
 	}
+elsif ($_[0]->{'socket'}) {
+	$d = '@@'.$_[0]->{'socket'};
+	}
 else {
 	$d = '*';
 	}
@@ -148,6 +155,33 @@ return ( 'debug', 'info', 'notice', 'warning',
 	 'err', 'crit', 'alert', 'emerg' );
 }
 
+# can_edit_log(&log)
+# Returns 1 if some log can be viewed/edited, 0 if not
+sub can_edit_log
+{
+return 1 if (!$access{'logs'});
+local @files = split(/\s+/, $access{'logs'});
+local $lf = $_[0]->{'file'} || $_[0]->{'pipe'} || $_[0]->{'host'} ||
+	    $_[0]->{'socket'} || ($_[0]->{'all'} ? "*" : "users");
+foreach $f (@files) {
+	return 1 if ($f eq $lf || &is_under_directory($f, $lf));
+	}
+return 0;
+}
+
+sub needs_m4
+{
+local $oldslash = $/;
+$/ = undef;
+open(CONF, $config{'syslog_conf'});
+local $conf1 = <CONF>;
+close(CONF);
+open(CONF, "$config{'m4_path'} $config{'syslog_conf'} |");
+local $conf2 = <CONF>;
+close(CONF);
+$/ = $oldslash;
+return $conf1 ne $conf2;
+}
 
 1;
 
