@@ -1,4 +1,5 @@
-require './time-lib.pl';
+
+require 'time-lib.pl';
 
 sub acl_security_form
 {
