@@ -59,12 +59,5 @@ close(WHO);
 return @rv;
 }
 
-# encrypt_password(password)
-sub encrypt_password
-{
-local $salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
-return crypt($_[0], $salt);
-}
-
 1;
 
