@@ -34,5 +34,29 @@ else {
 	}
 }
 
+# list_diffs(&action)
+sub list_diffs
+{
+local $i = 0;
+local @rv;
+while(open(DIFF, "$ENV{'WEBMIN_VAR'}/diffs/$act->{'id'}.$i")) {
+        local ($type, $object, $diff);
+        local $line = <DIFF>;
+        while(<DIFF>) { $diff .= $_; }
+        close(DIFF);
+	if ($line =~ /^(\/.*)/) {
+                $type = 'modify'; $object = $1;
+                }
+        elsif ($line =~ /^(\S+)\s+(.*)/) {
+                $type = $1; $object = $2;
+                }
+	push(@rv, { 'type' => $type,
+		    'object' => $object,
+		    'diff' => $diff } );
+	$i++;
+	}
+return @rv;
+}
+
 1;
 
